@@ -123,7 +123,7 @@
 						header("Pragma: public");
 						header("Expires: 0"); // set expiration time
 						header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
-						header("Content-Disposition: " . $openType . "; filename=\"" . $filename . "\"");
+						header("Content-Disposition: " . $openType . "; filename=\"" . browserDependingHTTPHeaderEncode($filename) . "\"");
 						header("Content-Transfer-Encoding: binary");
 						
 						// Set content type header
@@ -160,7 +160,7 @@
 					header("Pragma: public");
 					header("Expires: 0"); // set expiration time
 					header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
-					header("Content-Disposition: " . $openType . "; filename=\"" . $fileinfo['name'] . "\"");
+					header("Content-Disposition: " . $openType . "; filename=\"" . browserDependingHTTPHeaderEncode($fileinfo['name']) . "\"");
 					header("Content-Transfer-Encoding: binary");
 					header("Content-Type: application/octet-stream");
 					header("Content-Length: " . filesize($tmpname));
