@@ -40,6 +40,9 @@ $version = trim(file_get_contents('version'));
 			<span id="loading-message" class="loading"><?= _('Loading Zarafa WebApp ...') ?></span>
 		</div>
 
+		<!-- Translations -->
+		<script type="text/javascript" src="index.php?version=<?=$version?>&amp;load=translations.js&amp;lang=<?=$GLOBALS["language"]->getSelected()?>"></script>
+
 		<!-- ExtJS & Thirdparty extensions-->
 		<?php	
 			$extjsFiles = $loader->getExtjsJavascriptFiles(DEBUG_LOADER);
@@ -47,8 +50,6 @@ $version = trim(file_get_contents('version'));
 		?>
 
 		<!-- Zarafa client stuff -->
-		<script type="text/javascript" src="index.php?version=<?=$version?>&load=translations.js&lang=<?=$GLOBALS["language"]->getSelected()?>"></script>
-
 		<?php
 			$webappFiles = $loader->getZarafaJavascriptFiles(DEBUG_LOADER, $extjsFiles);
 			$loader->printFiles($webappFiles, $jsTemplate);
@@ -63,9 +64,6 @@ $version = trim(file_get_contents('version'));
 		<script type="text/javascript">
 			settings = <?= JSON::Encode($GLOBALS["settings"]->getJSON()); ?>;
 		</script>
-		<script type="text/javascript">
-			properties = <?= JSON::Encode($GLOBALS["properties"]->getJSON()); ?>;
-		</script>
 		<script type="text/javascript">
 			languages = <?= JSON::Encode($GLOBALS["language"]->getJSON()); ?>;
 		</script>
@@ -78,7 +76,7 @@ $version = trim(file_get_contents('version'));
 				email_address		: "<?= addslashes($GLOBALS["mapisession"]->getEmailAddress()) ?>",
 				smtp_address		: "<?= addslashes($GLOBALS["mapisession"]->getSMTPAddress()) ?>",
 				search_key		: "<?= bin2hex($GLOBALS["mapisession"]->getSearchKey()) ?>",
-				sessionid		: "<?= session_id() ?>"
+				sessionid		: "<?= $GLOBALS["mapisession"]->getSessionID() ?>"
 			};
 		</script>
 
@@ -90,5 +88,18 @@ $version = trim(file_get_contents('version'));
 				svn			: "<?= DEBUG_LOADER === LOAD_SOURCE ? svnversion() : '' ?>"
 			};
 		</script>
+
+		<script type="text/javascript">
+			serverconfig = {
+				enable_advanced_settings	: <?= ENABLE_ADVANCED_SETTINGS ? 'true' : 'false' ?>,
+				max_attachments			: undefined,
+				max_attachment_size		: <?= getMaxUploadSize() ?>,
+				max_attachment_total_size	: undefined
+			};
+		</script>
+
+		<script type="text/javascript">
+			Ext.onReady(Zarafa.loadWebclient, Zarafa);
+		</script>
 	</body>
 </html>
