@@ -1,14 +1,13 @@
 <?php
-
-if(!defined('CONFIG_CHECK')) define('CONFIG_CHECK', TRUE);
-if(!defined('CONFIG_CHECK_COOKIES_HTTP')) define('CONFIG_CHECK_COOKIES_HTTP', FALSE);
-if(!defined('CONFIG_CHECK_COOKIES_SSL')) define('CONFIG_CHECK_COOKIES_SSL', FALSE);
-
 /**
  * This file is used to set configuration options to a default value that have 
  * not been set in the config.php.Each definition of a configuration value must 
  * be preceeded by "if(!defined('KEY'))"
  */
+if(!defined('CONFIG_CHECK')) define('CONFIG_CHECK', TRUE);
+if(!defined('CONFIG_CHECK_COOKIES_HTTP')) define('CONFIG_CHECK_COOKIES_HTTP', FALSE);
+if(!defined('CONFIG_CHECK_COOKIES_SSL')) define('CONFIG_CHECK_COOKIES_SSL', FALSE);
+
 if(!defined('STATE_FILE_MAX_LIFETIME')) define('STATE_FILE_MAX_LIFETIME', 28*60*60);
 if(!defined('UPLOADED_ATTACHMENT_MAX_LIFETIME')) define('UPLOADED_ATTACHMENT_MAX_LIFETIME', 6*60*60);
 if(!defined('DISABLE_FULL_CONTACTLIST_THRESHOLD')) define('DISABLE_FULL_CONTACTLIST_THRESHOLD', -1);
@@ -28,6 +27,11 @@ if(!defined('DISABLE_HTMLBODY_FILTER')) define('DISABLE_HTMLBODY_FILTER', false)
  */
 if(!defined('DISABLE_REMOTE_USER_LOGIN')) define('DISABLE_REMOTE_USER_LOGIN', false);
 
+/**
+ * When set to true this disables the welcome screen to be shown for first time users.
+ */
+if(!defined('DISABLE_WELCOME_SCREEN')) define('DISABLE_WELCOME_SCREEN', false);
+
 /**
  * By default we won't disable the FULL GAB, as it is a performance option
  * which, when enabled, prevents the full GAB to be loaded'
@@ -47,11 +51,14 @@ if(!defined('ABITEMDETAILS_MAX_NUM_DISTLIST_MEMBERS')) define('ABITEMDETAILS_MAX
  */
 if(!defined('ENABLE_DIRECT_BOOKING')) define('ENABLE_DIRECT_BOOKING', true);
 
-if(!defined('ENABLED_LANGUAGES')) define("ENABLED_LANGUAGES", "de_DE;en_GB;en_US;es_ES;fi_FI;fr_FR;he_IL;hu_HU;it_IT;nb_NO;nl_NL;pl_PL;pt_BR;ru_RU;zh_CN");
+if(!defined('ENABLED_LANGUAGES')) define("ENABLED_LANGUAGES", "cs_CZ;de_DE;en_GB;en_US;es_ES;fi_FI;fr_FR;he_IL;hu_HU;it_IT;nb_NO;nl_NL;pl_PL;pt_BR;ru_RU;zh_CN;zh_TW");
 
 if(!defined('ENABLE_PLUGINS')) define('ENABLE_PLUGINS', true);
 if(!defined('PATH_PLUGIN_CONFIG_DIR')) define('PATH_PLUGIN_CONFIG_DIR', PATH_PLUGIN_DIR);
 
+// Disable/enabled advanced settings
+if(!defined('ENABLE_ADVANCED_SETTINGS')) define('ENABLE_ADVANCED_SETTINGS', false);
+
 /*
  * The following options are taken from the debug.php
  */
