@@ -19,11 +19,12 @@
 	// Start the session
 	session_name(COOKIE_NAME);
 
-	if($_POST && array_key_exists(COOKIE_NAME, $_POST))
+	if($_POST && array_key_exists(COOKIE_NAME, $_POST)) {
 		session_id($_POST[COOKIE_NAME]);
+	}
 
 	session_start();
-	
+
 	// check if config is correct
 	if (defined("CONFIG_CHECK")){
 		include("server/class.configcheck.php");
@@ -52,7 +53,6 @@
 	
 	require("server/core/class.settings.php");
 	require("server/core/class.language.php");
-	include("server/core/class.properties.php");
 
 	require("server/core/class.state.php");
 	require("server/core/class.attachmentstate.php");
@@ -60,47 +60,89 @@
 	require("server/core/class.pluginmanager.php");
 	require("server/core/class.plugin.php");
 
-	// Destroy session if an user loggs out
-	if($_GET && array_key_exists("logout", $_GET)) {
+	// Determine the action, use 'null' as default value
+	// so we will recognize arguments without value properly.
+	$logout = get('logout', null, false, STRING_REGEX) !== null;
+	$logon = get('logon', null, false, STRING_REGEX) !== null;
+	$load = get('load', false, false, FILENAME_REGEX);
+
+	// Returns true if the given $load argument
+	// requires a validated session.
+	function loadNeedsSession($load)
+	{
+		return !empty($load) && $load !== 'logon' && $load !== 'translations.js'; 
+	}
+
+	// Destroy session
+	function destroySession()
+	{
 		$_SESSION = array();
-		
+
 		if (isset($_COOKIE[session_name()])) {
 			setcookie(session_name(), '', time()-42000, '/');
 		}
+
 		session_destroy();
+	}
 
-		header("Location: index.php", true, 303);
+	if ($logout) {
+		// The user requests to logout. We should destroy the
+		// session, and redirect the user to the logon page.
+		$actionURI = '?load=logon';
+
+		destroySession();
+
+		$user = get('user', false, false, STRING_REGEX);
+		if ($user) {
+			$actionURI .= '&user=' . rawurlencode($user);
+		}
+
+		// Redirect the user, this will reload the page
+		// and request the logon page.
+		header('Location: index.php' . $actionURI, true, 303);
 		exit;
-	}
+	} else if ($logon) {
+		// The user requested to logon. Check if credentials were provided
+		// or if the a remote user login is possible.
+		$username = ($_POST && array_key_exists('username', $_POST)) ? $_POST['username'] : '';
+		$password = ($_POST && array_key_exists('password', $_POST)) ? $_POST['password'] : '';
 
-	// Set the session variables if it is posted
-	if($_POST && array_key_exists("username", $_POST) && array_key_exists("password", $_POST)) {
-		$_SESSION["username"] = $_POST["username"];
-		$_SESSION["password"] = $_POST["password"];
-	}
-	if(!DISABLE_REMOTE_USER_LOGIN){
-		// REMOTE_USER is set when apache has authenticated the user
-		if( ! $_POST && $_SERVER && array_key_exists("REMOTE_USER", $_SERVER)) {
-			$_SESSION["username"] = $_SERVER['REMOTE_USER'];
-			if (LOGINNAME_STRIP_DOMAIN) {
-				$_SESSION["username"] = ereg_replace('@.*', '', $_SESSION["username"]);
+		if (isset($_SESSION['username']) && $_SESSION['username'] !== $username) {
+			$hresult = MAPI_E_INVALID_WORKSTATION_ACCOUNT;
+			// Logon failed because a session for another user already exists.
+			// We force the user back to the logon page.
+			$load = 'logon';
+		} else if (!empty($username) && !empty($password)) {
+			// Set the session variables if it is posted
+			$_SESSION['username'] = $username;
+			$_SESSION['password'] = $password;
+		}
+
+		if (!DISABLE_REMOTE_USER_LOGIN){
+			// REMOTE_USER is set when apache has authenticated the user
+			if (!$_POST && $_SERVER && array_key_exists('REMOTE_USER', $_SERVER)) {
+				$_SESSION['username'] = $_SERVER['REMOTE_USER'];
+				if (LOGINNAME_STRIP_DOMAIN) {
+					$_SESSION['username'] = ereg_replace('@.*', '', $_SESSION['username']);
+				}
+				$_SESSION["password"] = '';
 			}
-			$_SESSION["password"] = "";
 		}
 	}
 
-
 	// Create global mapi object. This object is used in many other files
-	$GLOBALS["mapisession"] = new MAPISession();
-	if (isset($_SESSION["username"]) && isset($_SESSION["password"])) {
+	$GLOBALS["mapisession"] = new MAPISession(session_id());
+
+	// We will only allow the logon when the sessionid in the GET arguments matches the
+	// sessionid as send in the cookie. Otherwise the cookie was somehow modified in the
+	// browser.
+	$sessionid = get('sessionid', '', false, ID_REGEX);
+	if (loadNeedsSession($load) && $sessionid !== $GLOBALS["mapisession"]->getSessionId()) {
+		$hresult = MAPI_E_INVALID_WORKSTATION_ACCOUNT;
+	} else if ($load !== 'logon' && isset($_SESSION["username"]) && isset($_SESSION["password"])) {
 		$sslcert_file = defined('SSLCERT_FILE') ? SSLCERT_FILE : null;
 		$sslcert_pass = defined('SSLCERT_PASS') ? SSLCERT_PASS : null;
 		$hresult = $GLOBALS["mapisession"]->logon($_SESSION["username"], $_SESSION["password"], DEFAULT_SERVER, $sslcert_file, $sslcert_pass);
-		if ($hresult != NOERROR){
-			// login failed, remove session
-			$_SESSION = array();
-			$_SESSION["hresult"] = $hresult;
-		}
 	}
 
 	// Check if user is authenticated
@@ -111,9 +153,6 @@
 		$GLOBALS['PluginManager'] = new PluginManager(ENABLE_PLUGINS);
 		$GLOBALS['PluginManager']->detectPlugins(DISABLED_PLUGINS_LIST);
 		$GLOBALS['PluginManager']->initPlugins(DEBUG_LOADER);
-
-		// Create global properties object
-		$GLOBALS["properties"] = new Properties();
 		
 		// Create globals settings object
 		$GLOBALS["settings"] = new Settings();
@@ -145,7 +184,7 @@
 		}
 
 		$GLOBALS["language"]->setLanguage($lang);
-		if($_GET && array_key_exists("logon", $_GET)) {
+		if ($logon) {
 			$GLOBALS['PluginManager']->triggerHook("server.index.login.success");
 			header("Location: index.php", true, 303);
 			exit;
@@ -155,16 +194,16 @@
 		header("X-Zarafa: " . trim(file_get_contents('version')));
 
 		// external files who need our login
-		if ($_GET && array_key_exists("load", $_GET)) {
-
-			switch ($_GET["load"]) {
+		if ($load) {
+			switch ($load) {
 				case "translations.js":
 					$GLOBALS['PluginManager']->triggerHook("server.index.load.jstranslations.before");
 					include("client/translations.js.php");
 					$GLOBALS['PluginManager']->triggerHook("server.index.load.jstranslations.after");
 					break;
 				case "custom":
-					$GLOBALS['PluginManager']->triggerHook("server.index.load.custom", array('name' => $_GET['name']));
+					$name = get('name', '', false, STRING_REGEX);
+					$GLOBALS['PluginManager']->triggerHook("server.index.load.custom", array('name' => $name));
 					break;
 				case "upload_attachment":
 					$GLOBALS['PluginManager']->triggerHook("server.index.load.upload_attachment.before");
@@ -181,6 +220,9 @@
 					include("client/download_message.php");
 					$GLOBALS['PluginManager']->triggerHook("server.index.load.download_message.after");
 					break;
+				case "logon":
+					include("client/login.php");
+					break;
 				default:
 					// These hooks are defined twice (also when no "load" argument is supplied)
 					$GLOBALS['PluginManager']->triggerHook("server.index.load.main.before");
@@ -192,7 +234,6 @@
 			$version = trim(file_get_contents('version'));
 			$data = array(
 				"settings" => $GLOBALS["settings"]->getJSON(),
-				"properties" => $GLOBALS["properties"]->getJSON(),
 				"languages" => $GLOBALS["language"]->getJSON(),
 				"user" => array(
 					"username" 	=> addslashes($GLOBALS["mapisession"]->getUserName()),
@@ -201,26 +242,48 @@
 					"email_address" => addslashes($GLOBALS["mapisession"]->getEmailAddress()),
 					"smtp_address" 	=> addslashes($GLOBALS["mapisession"]->getSMTPAddress()),
 					"search_key" 	=> bin2hex($GLOBALS["mapisession"]->getSearchKey()),
-					"sessionid" 	=> session_id()
+					"sessionid" 	=> $GLOBALS["mapisession"]->getSessionID()
 				),
 				"version" => array(
 					"webapp"	=> $version,
 					"zcp"		=> phpversion('mapi'),
 					"server"	=> DEBUG_SHOW_SERVER ? DEBUG_SERVER_ADDRESS : '',
 					"svn"		=> DEBUG_LOADER === LOAD_SOURCE ? svnversion() : ''
+				),
+				"server" => array(
+					"enable_advanced_settings"	=> !!ENABLE_ADVANCED_SETTINGS,
+					"max_attachments"		=> null,
+					"max_attachment_size"		=> getMaxUploadSize(),
+	                                "max_attachment_total_size"	=> null
 				)
 			);
 			echo JSON::Encode(array("zarafa" => $data));
 		} else if ($_GET && array_key_exists("verify", $_GET)) {
 			
-			$user = $_GET["verify"];
+			$user = get('verify', false, false, STRING_REGEX);
 			
 			if($user == $_SESSION['username'])
 				print "1";
 			else
 				print "0";
 			
+		} else if (!DISABLE_WELCOME_SCREEN && $GLOBALS["settings"]->get("zarafa/v1/main/show_welcome") !== false) {
+			// The user wants to logon, but he never did this before (or was using
+			// an older version of WebApp, before the Welcome screen was introduced),
+			// we will show a Welcome to WebApp screen where some default settings
+			// might be configured
+
+			// These hooks are defined twice (also when there is a "load" argument supplied)
+			$GLOBALS['PluginManager']->triggerHook("server.index.load.welcome.before");
+			// Include welcome page
+			include("client/welcome.php");
+			$GLOBALS['PluginManager']->triggerHook("server.index.load.welcome.after");
 		} else {
+			// Set the show_welcome to true, so that when the admin is changing the 
+			// DISABLE_WELCOME_SCREEN option to false after some time, the users who are already 
+			// using the WebApp are not bothered with the Welcome Screen.
+			$GLOBALS["settings"]->set("zarafa/v1/main/show_welcome", false);
+
 			// Clean up old state files in tmp/session/
 			$state = new State("index");
 			$state->clean();
@@ -241,12 +304,35 @@
 
 		// Save the settings to the MAPI store
 		$GLOBALS['settings']->saveSettings();
-
-	} else { // Not authenticated, goto login page
-		// Create global language object
+	} else if (empty($load) || $load === 'logon') {
+		// We are not authenticated, and the requested page was either
+		// the logon page itself, or the default page. In either case
+		// we can redirect the user back to the logon page.
 		$GLOBALS["language"] = new Language();
 		$GLOBALS["language"]->setLanguage(LANG);
 
+		if(isset($GLOBALS["hresult"])) {
+			switch($GLOBALS["hresult"]) {
+				case MAPI_E_LOGON_FAILED:
+				case MAPI_E_UNCONFIGURED:
+					// destroy the session so another login attempt will not use preserved data
+					destroySession();
+					break;
+			}
+		}
+
+		// NOTE: We have saved the $hresult in the $GLOBALS
+		// object, the login.php will obtain the code from there.
 		include("client/login.php");
+	} else if ($hresult === MAPI_E_NETWORK_ERROR) {
+		// The user is not logged in because the zarafa-server could not be reached.
+		// Return a HTTP 503 error so the client can act upon this event correctly.
+		header('HTTP/1.1 503 Service unavailable');
+		header("X-Zarafa-Hresult: " . get_mapi_error_name($hresult));
+	} else {
+		// The session expired, or the user is otherwise not logged on.
+		// Return a HTTP 401 error so the client can act upon this event correctly.
+		header('HTTP/1.1 401 Unauthorized');
+		header("X-Zarafa-Hresult: " . get_mapi_error_name($hresult));
 	}
 ?>
