@@ -1,13 +1,18 @@
 <?php
 require_once("class.xmlparser.php");
 
+define("TYPE_PLUGIN", 1);
+define("TYPE_MODULE", 2);
+define("TYPE_CONFIG", 3);
+define("TYPE_NOTIFIER", 4);
+
 /**
-* Managing component for all plugins
-*
-* This class handles all the plugin interaction with the webaccess on the server side.
-*
-* @package core
-*/
+ * Managing component for all plugins
+ *
+ * This class handles all the plugin interaction with the webaccess on the server side.
+ *
+ * @package core
+ */
 class PluginManager
 {
 	// True if the Plugin framework is enabled
@@ -34,12 +39,38 @@ class PluginManager
 	 */
 	var $modules;
 
+	/**
+	 * List of all provided notifiers
+	 * [notifiername] = notifierFile
+	 */
+	var $notifiers;
+
 	/**
 	 * List of sessiondata from plugins. 
 	 * [pluginname] = sessiondata
 	 */
 	var $sessionData;
 
+	/**
+	 * Mapping for the XML 'load' attribute values
+	 * to the corresponding define.
+	 */
+	var $loadMap = Array(
+		'release' => LOAD_RELEASE,
+		'debug' => LOAD_DEBUG,
+		'source' => LOAD_SOURCE
+	);
+
+	/**
+	 * Mapping for the XML 'type' attribute values
+	 * to the corresponding define.
+	 */
+	var $typeMap = Array(
+		'plugin' => TYPE_PLUGIN,
+		'module' => TYPE_MODULE,
+		'notifier' => TYPE_NOTIFIER
+	);
+
 	/**
 	 * Constructor
 	 */
@@ -50,6 +81,7 @@ class PluginManager
 		$this->hooks = Array();
 		$this->plugins = Array();
 		$this->modules = Array();
+		$this->notifiers = Array();
 		$this->sessionData = false;
 	} 
 
@@ -83,13 +115,14 @@ class PluginManager
 		$pluginState = new State('plugin');
 		$pluginState->open();
 		$this->plugindata = $pluginState->read("plugindata");
-		$disabledPlugins = Array();
-		if (!empty($disabled)) {
-			$disabledPlugins = explode(';', $disabled);
-		}
 
 		// If no plugindata has been stored yet, get it from the plugins dir.
-		if(!$this->plugindata || DEBUG_PLUGINS){
+		if(!$this->plugindata || DEBUG_PLUGINS_DISABLE_CACHE){
+			$disabledPlugins = Array();
+			if (!empty($disabled)) {
+				$disabledPlugins = explode(';', $disabled);
+			}
+
 			$this->plugindata = Array();
 			$pluginsdir = opendir(PATH_PLUGIN_DIR);
 			if($pluginsdir) {
@@ -108,6 +141,7 @@ class PluginManager
 			// Write the plugindata to the state
 			$pluginState->write("plugindata", $this->plugindata);
 		}
+
 		// Free the state again.
 		$pluginState->close();
 	}
@@ -126,19 +160,24 @@ class PluginManager
 			return false;
 		}
 
-		// Inlcude the root files of all the plugins and instantiate the plugin
-		foreach($this->plugindata as $plugName => $plugData){
-			$this->includePluginServerFiles($plugName, $load);
+		$files = $this->getServerFiles($load);
+		foreach ($files['server'] as $file) {
+			include_once($file);
+		}
 
-			if(class_exists('Plugin' . $plugName)){
-				$pluginClassName = 'Plugin' . $plugName;
+		// Inlcude the root files of all the plugins and instantiate the plugin
+		foreach($this->plugindata as $plugName => &$plugData){
+			$pluginClassName = 'Plugin' . $plugName;
+			if(class_exists($pluginClassName)){
 				$this->plugins[$plugName] = new $pluginClassName;
 				$this->plugins[$plugName]->setPluginName($plugName);
 				$this->plugins[$plugName]->init();
 			}
 		}
+		unset($plugData);
 
-		$this->modules = $this->getModuleFiles($load);
+		$this->modules = $files['modules'];
+		$this->notifiers = $files['notifiers'];
 	}
 
 	/**
@@ -218,23 +257,6 @@ class PluginManager
 		}
 	}
 	
-	/**
-	 * includePluginServerFiles
-	 * 
-	 * Include the server plugin files. 
-	 * 
-	 * @param $pluginname string Identifier of the plugin
-	 * @param number $load One of LOAD_RELEASE, LOAD_DEBUG, LOAD_SOURCE. This will filter
-	 * the files based on the 'load' attribute. 
-	 */
-	function includePluginServerFiles($pluginname, $load = LOAD_RELEASE){
-		$files = $this->getServerFiles($pluginname, $load);
-
-		foreach ($files as $file) {
-			include_once($file);
-		}
-	}
-
 	/**
 	 * pluginExists
 	 * 
@@ -268,6 +290,23 @@ class PluginManager
 		return false;
 	}
 
+	/**
+	 * getNotifierFilePath
+	 *
+	 * Obtain the filepath of the given notifiername
+	 *
+	 * @param $notifiername string Identifier of the notifiername
+	 * @return string The path to the file for the notifier
+	 */
+	function getNotifierFilePath($notifiername)
+	{
+		if (isset($this->notifiers[$notifiername])) {
+			return $this->notifiers[$notifiername];
+		}
+
+		return false;
+	}
+
 	/**
 	 * registerHook
 	 * 
@@ -298,83 +337,6 @@ class PluginManager
 		return $data;
 	}
 
-	/**
-	 * getModuleFilesForComponent
-	 *
-	 * Called by getModuleFiles() to return the list of files which are provided
-	 * for the given component in a particular plugin.
-	 * The paths which are returned start at the root of the webapp.
-	 *
-	 * This function might call itself recursively if it couldn't find any files for
-	 * the given $load type. If no 'source' files are found, it will obtain the 'debug'
-	 * files, if that too files it will fallback to 'release' files. If the latter is
-	 * not found either, no files are returned.
-	 *
-	 * @param String $pluginname The name of the plugin (this is used in the pathname)
-	 * @param Array $component The component to read the modulefiles from
-	 * @param String $load The load type by which to filter (one of 'release', 'debug' and 'source')
-	 * @return array list of paths to the files in this component
-	 */
-	function getModuleFilesForComponent($pluginname, $component, $load)
-	{
-		$componentfiles = Array();
-
-		for($i = 0, $len = count($component['serverfiles']); $i < $len; $i++){
-			if ($component['serverfiles'][$i]['type'] === 'module' && $component['serverfiles'][$i]['load'] === $load) {
-				if (!empty($component['serverfiles'][$i]['module'])) {
-					$componentfiles[ $component['serverfiles'][$i]['module'] ] = PATH_PLUGIN_DIR.'/'.$pluginname.'/'.$component['serverfiles'][$i]['file'];
-				}
-			}
-		}
-
-		if (empty($componentfiles)) {
-			if ($load === 'source') {
-				$componentfiles = $this->getModuleFilesForComponent($pluginname, $component, 'debug');
-			} else if ($load === 'debug') {
-				$componentfiles = $this->getModuleFilesForComponent($pluginname, $component, 'release');
-			}
-			// else tough luck, at least release should be present
-		}
-
-		return $componentfiles;
-	}
-
-	/**
-	 * getModuleFiles
-	 * 
-	 * Returning an array of paths to files that need to be included.
-	 * The paths which are returned start at the root of the webapp.
-	 *
-	 * This calls getModuleFilesForComponent() to obtain the files
-	 * for each component inside the requested plugin
-	 *
-	 * @param string $pluginname The name of the plugin for which the module files are requested.
-	 * @param number $load One of LOAD_RELEASE, LOAD_DEBUG, LOAD_SOURCE. This will filter
-	 * the files based on the 'load' attribute. 
-	 * @return array List of paths to files.
-	 */
-	function getModuleFiles($load = LOAD_RELEASE)
-	{
-		$files = Array();
-
-		if ($load === LOAD_RELEASE) {
-			$load = 'release';
-		} else if ($load === LOAD_DEBUG) {
-			$load = 'debug';
-		} else {
-			$load = 'source';
-		}
-
-		foreach($this->plugindata as $pluginname => $plugin){
-			foreach($plugin['components'] as $component) {
-				$componentfiles = $this->getModuleFilesForComponent($pluginname, $component, $load); 
-				$files = array_merge($files, $componentfiles);
-			}
-		}
-
-		return $files;
-	}
-
 	/**
 	 * getServerFilesForComponent
 	 *
@@ -389,29 +351,38 @@ class PluginManager
 	 *
 	 * @param String $pluginname The name of the plugin (this is used in the pathname)
 	 * @param Array $component The component to read the serverfiles from
-	 * @param String $load The load type by which to filter (one of 'release', 'debug' and 'source')
+	 * @param number $load One of LOAD_RELEASE, LOAD_DEBUG, LOAD_SOURCE. This will filter
+	 * the files based on the 'load' attribute. 
 	 * @return array list of paths to the files in this component
 	 */
 	function getServerFilesForComponent($pluginname, $component, $load)
 	{
-		$componentfiles = Array();
+		$componentfiles = Array(
+			'server' => Array(),
+			'modules' => Array(),
+			'notifiers' => Array()
+		);
 
-		for($i = 0, $len = count($component['serverfiles']); $i < $len; $i++){
+		foreach ($component['serverfiles'][$load] as &$file) {
 			// FIXME: With WA-3351 config files should be handled differently, and this
 			// if-statement must be updated.
-			if ($component['serverfiles'][$i]['type'] === 'config' || ($component['serverfiles'][$i]['type'] === 'plugin' && $component['serverfiles'][$i]['load'] === $load)) {
-				$componentfiles[] = PATH_PLUGIN_DIR.'/'.$pluginname.'/'.$component['serverfiles'][$i]['file'];
+			switch ($file['type']) {
+				case TYPE_CONFIG:
+					$componentfiles['server'][] = PATH_PLUGIN_CONFIG_DIR . '/' . $pluginname . '/' . $file['file'];
+					break;
+				case TYPE_PLUGIN:
+					$componentfiles['server'][] = PATH_PLUGIN_DIR . '/' . $pluginname . '/' . $file['file'];
+					break;
+				case TYPE_MODULE:
+					$componentfiles['modules'][ $file['module'] ] = PATH_PLUGIN_DIR.'/'.$pluginname.'/'.$file['file'];
+					break;
+				case TYPE_NOTIFIER:
+					$componentfiles['notifiers'][ $file['notifier'] ] = PATH_PLUGIN_DIR.'/'.$pluginname.'/'.$file['file'];
+					break;
 			}
-		}
 
-		if (empty($componentfiles)) {
-			if ($load === 'source') {
-				$componentfiles = $this->getServerFilesForComponent($pluginname, $component, 'debug');
-			} else if ($load === 'debug') {
-				$componentfiles = $this->getServerFilesForComponent($pluginname, $component, 'release');
-			}
-			// else tough luck, at least release should be present
 		}
+		unset($file);
 
 		return $componentfiles;
 	}
@@ -430,23 +401,34 @@ class PluginManager
 	 * the files based on the 'load' attribute. 
 	 * @return array List of paths to files.
 	 */
-	function getServerFiles($pluginname, $load = LOAD_RELEASE)
+	function getServerFiles($load = LOAD_RELEASE)
 	{
-		$files = Array();
+		$files = Array(
+			'server' => Array(),
+			'modules' => Array(),
+			'notifiers' => Array()
+		);
 
-		if ($load === LOAD_RELEASE) {
-			$load = 'release';
-		} else if ($load === LOAD_DEBUG) {
-			$load = 'debug';
-		} else {
-			$load = 'source';
-		}
+		foreach($this->plugindata as $pluginname => &$plugin){
+			foreach($plugin['components'] as &$component) {
+				if (!empty($component['serverfiles'][$load])) {
+					$componentfiles = $this->getServerFilesForComponent($pluginname, $component, $load);
+				} else if ($load === LOAD_SOURCE && !empty($component['serverfiles'][LOAD_DEBUG])) {
+					$componentfiles = $this->getServerFilesForComponent($pluginname, $component, LOAD_DEBUG);
+				} else if ($load !== LOAD_RELEASE && !empty($component['serverfiles'][LOAD_RELEASE])) {
+					$componentfiles = $this->getServerFilesForComponent($pluginname, $component, LOAD_RELEASE);
+				} // else tough luck, at least release should be present
 
-		$plugin = $this->plugindata[$pluginname];
-		foreach ($plugin['components'] as $component) {
-			$componentfiles = $this->getServerFilesForComponent($pluginname, $component, $load);
-			$files = array_merge($files, $componentfiles);
+				if (isset($componentfiles)) {
+					$files['server'] = array_merge($files['server'], $componentfiles['server']);
+					$files['modules'] = array_merge($files['modules'], $componentfiles['modules']);
+					$files['notifiers'] = array_merge($files['notifiers'], $componentfiles['notifiers']);
+					unset($componentfiles);
+				}
+			}
+			unset($component);
 		}
+		unset($plugin);
 
 		return $files;
 	}
@@ -465,27 +447,18 @@ class PluginManager
 	 *
 	 * @param String $pluginname The name of the plugin (this is used in the pathname)
 	 * @param Array $component The component to read the clientfiles from
-	 * @param String $load The load type by which to filter (one of 'release', 'debug' and 'source')
+	 * @param number $load One of LOAD_RELEASE, LOAD_DEBUG, LOAD_SOURCE. This will filter
+	 * the files based on the 'load' attribute. 
 	 * @return array list of paths to the files in this component
 	 */
 	function getClientFilesForComponent($pluginname, $component, $load)
 	{
 		$componentfiles = Array();
 
-		for($i = 0, $len = count($component['clientfiles']); $i < $len; $i++){
-			if ($component['clientfiles'][$i]['load'] === $load) {
-				$componentfiles[] = PATH_PLUGIN_DIR.'/'.$pluginname.'/'.$component['clientfiles'][$i]['file'];
-			}
-		}
-
-		if (empty($componentfiles)) {
-			if ($load === 'source') {
-				$componentfiles = $this->getClientFilesForComponent($pluginname, $component, 'debug');
-			} else if ($load === 'debug') {
-				$componentfiles = $this->getClientFilesForComponent($pluginname, $component, 'release');
-			}
-			// else tough luck, at least release should be present
+		foreach ($component['clientfiles'][$load] as &$file) {
+			$componentfiles[] = PATH_PLUGIN_DIR.'/'.$pluginname.'/'.$file['file'];
 		}
+		unset($file);
 
 		return $componentfiles;
 	}
@@ -506,20 +479,24 @@ class PluginManager
 	function getClientFiles($load = LOAD_RELEASE){
 		$files = Array();
 
-		if ($load === LOAD_RELEASE) {
-			$load = 'release';
-		} else if ($load === LOAD_DEBUG) {
-			$load = 'debug';
-		} else {
-			$load = 'source';
-		}
+		foreach($this->plugindata as $pluginname => &$plugin){
+			foreach($plugin['components'] as &$component) {
+				if (!empty($component['clientfiles'][$load])) {
+					$componentfiles = $this->getClientFilesForComponent($pluginname, $component, $load);
+				} else if ($load === LOAD_SOURCE && !empty($component['clientfiles'][LOAD_DEBUG])) {
+					$componentfiles = $this->getClientFilesForComponent($pluginname, $component, LOAD_DEBUG);
+				} else if ($load !== LOAD_RELEASE && !empty($component['clientfiles'][LOAD_RELEASE])) {
+					$componentfiles = $this->getClientFilesForComponent($pluginname, $component, LOAD_RELEASE);
+				} // else tough luck, at least release should be present
 
-		foreach($this->plugindata as $pluginname => $plugin){
-			foreach($plugin['components'] as $component) {
-				$componentfiles = $this->getClientFilesForComponent($pluginname, $component, $load); 
-				$files = array_merge($files, $componentfiles);
+				if (isset($componentfiles)) {
+					$files = array_merge($files, $componentfiles);
+					unset($componentfiles);
+				}
 			}
+			unset($component);
 		}
+		unset($plugin);
 
 		return $files;
 	}
@@ -538,29 +515,20 @@ class PluginManager
 	 *
 	 * @param String $pluginname The name of the plugin (this is used in the pathname)
 	 * @param Array $component The component to read the resourcefiles from
-	 * @param String $load The load type by which to filter (one of 'release', 'debug' and 'source')
+	 * @param number $load One of LOAD_RELEASE, LOAD_DEBUG, LOAD_SOURCE. This will filter
+	 * the files based on the 'load' attribute. 
 	 * @return array list of paths to the files in this component
 	 */
 	function getResourceFilesForComponent($pluginname, $component, $load)
 	{
-		$pluginfiles = Array();
+		$componentfiles = Array();
 
-		for($i = 0, $len = count($component['resourcefiles']); $i < $len; $i++){
-			if ($component['resourcefiles'][$i]['load'] === $load) {
-				$pluginfiles[] = PATH_PLUGIN_DIR.'/'.$pluginname.'/'.$component['resourcefiles'][$i]['file'];
-			}
+		foreach ($component['resourcefiles'][$load] as &$file) {
+			$componentfiles[] = PATH_PLUGIN_DIR.'/'.$pluginname.'/'.$file['file'];
 		}
+		unset($file);
 
-		if (empty($pluginfiles)) {
-			if ($load === 'source') {
-				$pluginfiles = $this->getResourceFilesForComponent($pluginname, $component, 'debug');
-			} else if ($load === 'debug') {
-				$pluginfiles = $this->getResourceFilesForComponent($pluginname, $component, 'release');
-			}
-			// else tough luck, at least release should be present
-		}
-
-		return $pluginfiles;
+		return $componentfiles;
 	}
 
 	/**
@@ -579,20 +547,25 @@ class PluginManager
 	function getResourceFiles($load = LOAD_RELEASE) {
 		$files = Array();
 
-		if ($load === LOAD_RELEASE) {
-			$load = 'release';
-		} else if ($load === LOAD_DEBUG) {
-			$load = 'debug';
-		} else {
-			$load = 'source';
-		}
+		foreach($this->plugindata as $pluginname => &$plugin){
+			foreach($plugin['components'] as &$component) {
+				if (!empty($component['resourcefiles'][$load])) {
+					$componentfiles = $this->getResourceFilesForComponent($pluginname, $component, $load);
+				} else if ($load === LOAD_SOURCE && !empty($component['resourcefiles'][LOAD_DEBUG])) {
+					$componentfiles = $this->getResourceFilesForComponent($pluginname, $component, LOAD_DEBUG);
+				} else if ($load !== LOAD_RELEASE && !empty($component['resourcefiles'][LOAD_RELEASE])) {
+					$componentfiles = $this->getResourceFilesForComponent($pluginname, $component, LOAD_RELEASE);
+				} // else tough luck, at least release should be present
 
-		foreach($this->plugindata as $pluginname => $plugin){
-			foreach($plugin['components'] as $component) {
-				$componentfiles = $this->getResourceFilesForComponent($pluginname, $component, $load); 
-				$files = array_merge($files, $componentfiles);
+				if (isset($componentfiles)) {
+					$files = array_merge($files, $componentfiles);
+					unset($componentfiles);
+				}
 			}
+			unset($component);
 		}
+		unset($plugin);
+
 		return $files;
 	}
 
@@ -606,7 +579,8 @@ class PluginManager
 	 */
 	function getTranslationFilePaths(){
 		$paths = Array();
-		foreach($this->plugindata as $pluginname => $plugin){
+
+		foreach($this->plugindata as $pluginname => &$plugin){
 			if ($plugin['translationsdir']) {
 				$translationPath =  PATH_PLUGIN_DIR.'/'.$pluginname.'/'.$plugin['translationsdir']['dir'];
 				if(is_dir($translationPath)){
@@ -614,6 +588,8 @@ class PluginManager
 				}
 			}
 		}
+		unset($plugin);
+
 		return $paths;
 	}
 
@@ -669,30 +645,49 @@ class PluginManager
 		if (isset($data['components']) && isset($data['components']['component'])) {
 			$components = $data['components']['component'];
 
-			foreach ($components as $index => $component) {
+			foreach ($components as $index => &$component) {
 				$componentdata = array(
-					'serverfiles' => Array(),
-					'clientfiles' => Array(),
-					'resourcefiles' => Array(),
+					'serverfiles' => Array(
+						LOAD_SOURCE => Array(),
+						LOAD_DEBUG => Array(),
+						LOAD_RELEASE => Array()
+					),
+					'clientfiles' => Array(
+						LOAD_SOURCE => Array(),
+						LOAD_DEBUG => Array(),
+						LOAD_RELEASE => Array()
+					),
+					'resourcefiles' => Array(
+						LOAD_SOURCE => Array(),
+						LOAD_DEBUG => Array(),
+						LOAD_RELEASE => Array()
+					),
 				);
 
 				if (isset($component['files'])) {
 					if (isset($component['files']['server']) && isset($component['files']['server']['serverfile'])) {
 						$newfiles = $this->getServerFileInfoFromXML($component['files']['server']['serverfile']);
-						$componentdata['serverfiles'] = array_merge($componentdata['serverfiles'], $newfiles);
+						$componentdata['serverfiles'][LOAD_SOURCE] = array_merge($componentdata['serverfiles'][LOAD_SOURCE], $newfiles[LOAD_SOURCE]);
+						$componentdata['serverfiles'][LOAD_DEBUG] = array_merge($componentdata['serverfiles'][LOAD_DEBUG], $newfiles[LOAD_DEBUG]);
+						$componentdata['serverfiles'][LOAD_RELEASE] = array_merge($componentdata['serverfiles'][LOAD_RELEASE], $newfiles[LOAD_RELEASE]);
 					}
 					if (isset($component['files']['client']) && isset($component['files']['client']['clientfile'])) {
 						$newfiles = $this->getClientFileInfoFromXML($component['files']['client']['clientfile']);
-						$componentdata['clientfiles'] = array_merge($componentdata['clientfiles'], $newfiles);
+						$componentdata['clientfiles'][LOAD_SOURCE] = array_merge($componentdata['clientfiles'][LOAD_SOURCE], $newfiles[LOAD_SOURCE]);
+						$componentdata['clientfiles'][LOAD_DEBUG] = array_merge($componentdata['clientfiles'][LOAD_DEBUG], $newfiles[LOAD_DEBUG]);
+						$componentdata['clientfiles'][LOAD_RELEASE] = array_merge($componentdata['clientfiles'][LOAD_RELEASE], $newfiles[LOAD_RELEASE]);
 					}
 					if (isset($component['files']['resources']) && isset($component['files']['resources']['resourcefile'])) {
 						$newfiles = $this->getResourceFileInfoFromXML($component['files']['resources']['resourcefile']);
-						$componentdata['resourcefiles'] = array_merge($componentdata['resourcefiles'], $newfiles);
+						$componentdata['resourcefiles'][LOAD_SOURCE] = array_merge($componentdata['resourcefiles'][LOAD_SOURCE], $newfiles[LOAD_SOURCE]);
+						$componentdata['resourcefiles'][LOAD_DEBUG] = array_merge($componentdata['resourcefiles'][LOAD_DEBUG], $newfiles[LOAD_DEBUG]);
+						$componentdata['resourcefiles'][LOAD_RELEASE] = array_merge($componentdata['resourcefiles'][LOAD_RELEASE], $newfiles[LOAD_RELEASE]);
 					}
 				}
 
 				$plugindata['components'][] = $componentdata;
 			}
+			unset($component);
 		} else {
 			if (DEBUG_PLUGINS) {
 				dump('[PLUGIN ERROR] Plugin manifest didn\t provide any components');
@@ -712,7 +707,11 @@ class PluginManager
 	 * @return array List of config files.
 	 */
 	function getConfigFileInfoFromXML($fileData) {
-		$files = Array();
+		$files = Array(
+			LOAD_SOURCE => Array(),
+			LOAD_DEBUG => Array(),
+			LOAD_RELEASE => Array()
+		);
 
 		for ($i = 0, $len = count($fileData); $i < $len; $i++) {
 			$filename = false;
@@ -728,11 +727,11 @@ class PluginManager
 			}
 
 			if ($filename) {
-				$files[] = Array(
+				$files[LOAD_RELEASE][] = Array(
 					'file' => $filename,
 					// FIXME: Remove in WA-3351
-					'type' => 'config',
-					'load' => 'release',
+					'type' => TYPE_CONFIG,
+					'load' => LOAD_RELEASE,
 					'module' => null,
 				);
 			}
@@ -758,12 +757,16 @@ class PluginManager
 	 * @return array List of server files.
 	 */
 	function getServerFileInfoFromXML($fileData) {
-		$files = Array();
+		$files = Array(
+			LOAD_SOURCE => Array(),
+			LOAD_DEBUG => Array(),
+			LOAD_RELEASE => Array()
+		);
 
 		for ($i = 0, $len = count($fileData); $i < $len; $i++) {
 			$filename = false;
-			$load = 'release';	// release | debug | source
-			$type = 'plugin';	// plugin | module
+			$load = LOAD_RELEASE;	// release | debug | source
+			$type = TYPE_PLUGIN;	// plugin | module | notifier
 			$module = null;
 
 			if (is_string($fileData[$i])) {
@@ -773,9 +776,11 @@ class PluginManager
 				if (isset($fileData[$i]['attributes'])) {
 					if (isset($fileData[$i]['attributes']['load'])) {
 						$load = $fileData[$i]['attributes']['load'];
+						$load = $this->loadMap[$load];
 					}
 					if (isset($fileData[$i]['attributes']['type'])) {
 						$type = $fileData[$i]['attributes']['type'];
+						$type = $this->typeMap[$type];
 					}
 					if (isset($fileData[$i]['attributes']['module'])) {
 						$module = $fileData[$i]['attributes']['module'];
@@ -788,7 +793,7 @@ class PluginManager
 			}
 
 			if ($filename) {
-				$files[] = Array(
+				$files[$load][] = Array(
 					'file' => $filename,
 					'type' => $type,
 					'load' => $load,
@@ -812,11 +817,15 @@ class PluginManager
 	 * @return array List of client files.
 	 */
 	function getClientFileInfoFromXML($fileData){
-		$files = Array();
+		$files = Array(
+			LOAD_SOURCE => Array(),
+			LOAD_DEBUG => Array(),
+			LOAD_RELEASE => Array()
+		);
 
 		for ($i = 0, $len = count($fileData); $i < $len; $i++) {
 			$filename = false;
-			$load = 'release';   // release | debug | source
+			$load = LOAD_RELEASE;   // release | debug | source
 
 			if (is_string($fileData[$i])) {
 				$filename = $fileData[$i];
@@ -825,6 +834,7 @@ class PluginManager
 				if (isset($fileData[$i]['attributes'])) {
 					if(isset($fileData[$i]['attributes']['load'])){
 						$load = $fileData[$i]['attributes']['load'];
+						$load = $this->loadMap[$load];
 					}
 				}
 			} else {
@@ -833,7 +843,7 @@ class PluginManager
 				}
 			}
 
-			$files[] = Array(
+			$files[$load][] = Array(
 				'file' => $filename,
 				'load' => $load,
 			);
@@ -854,11 +864,15 @@ class PluginManager
 	 * @return array List of resource files.
 	 */
 	function getResourceFileInfoFromXML($fileData) {
-		$files = Array();
+		$files = Array(
+			LOAD_SOURCE => Array(),
+			LOAD_DEBUG => Array(),
+			LOAD_RELEASE => Array()
+		);
 
 		for ($i = 0, $len = count($fileData); $i < $len; $i++) {
 			$filename = false;
-			$load = 'release';   // release | debug | source
+			$load = LOAD_RELEASE;   // release | debug | source
 
 			if (is_string($fileData[$i])) {
 				$filename = $fileData[$i];
@@ -867,6 +881,7 @@ class PluginManager
 				if (isset($fileData[$i]['attributes'])) {
 					if(isset($fileData[$i]['attributes']['load'])){
 						$load = $fileData[$i]['attributes']['load'];
+						$load = $this->loadMap[$load];
 					}
 				}
 			} else {
@@ -875,7 +890,7 @@ class PluginManager
 				}
 			}
 
-			$files[] = Array(
+			$files[$load][] = Array(
 				'file' => $filename,
 				'load' => $load,
 			);
