@@ -100,46 +100,6 @@
 			$this->mapping = array();
 		}
 
-		/**
-		 * Function will retrieve all the properties for all the loaded stores in mapisession
-		 * and will create a json array object to send it to client side.
-		 * @return String json encoded php array for all properties.
-		 */
-		function getJSON()
-		{
-			$this->Init();
-
-			$output = array();
-
-			// @FIXME this depends on WA-2418
-			// get list of all loaded stores
-			//$stores = $GLOBALS['mapisession']->getAllMessageStores();
-
-			/*foreach($stores as $entryId => $storeObj) {
-				$output[bin2hex($entryid)] = array(
-					'appointment' => $this->getAppointmentProperties($storeObj),
-					'contact' => $this->getContactProperties($storeObj),
-					'distlist' => $this->getDistListProperties($storeObj),
-					'mail' => $this->getMailProperties($storeObj),
-					'note' => $this->getStickyNoteProperties($storeObj),
-					'task' => $this->getTaskProperties($storeObj)
-				);
-			}*/
-
-			$output[$this->storeMapping] = array(
-				'calendar' => $this->getAppointmentProperties(),
-				'contact' => $this->getContactProperties(),
-				'distlist' => $this->getDistListProperties(),
-				'mail' => $this->getMailProperties(),
-				'note' => $this->getStickyNoteProperties(),
-				'task' => $this->getTaskProperties()
-			);
-
-			// @TODO this implementation should be extended to add more named properties by plugins also
-
-			return $output;
-		}
-
 		/**
 		 * Returns the properties for Recipients in a message
 		 * @return array properties for Recipient.
@@ -307,6 +267,10 @@
 				$properties["creation_time"] = PR_CREATION_TIME;
 				$properties["request_sent"] = "PT_BOOLEAN:PSETID_Appointment:0x8229";
 
+				// Meeting properties
+				$properties["goid"] = "PT_BINARY:PSETID_Meeting:0x3";
+				$properties["goid2"] = "PT_BINARY:PSETID_Meeting:0x23";
+
 				// Conflict properties
 				$properties["conflictitems"] = PR_CONFLICT_ITEMS;
 
@@ -336,6 +300,9 @@
 		{
 			$properties = $this->getAppointmentProperties();
 
+			unset($properties["goid"]);
+			unset($properties["goid2"]);
+
 			return $properties;
 		}
 
@@ -933,7 +900,6 @@
 			unset($properties['proposed_start_date']);
 			unset($properties['proposed_end_date']);
 			unset($properties['proposed_duration']);
-			unset($properties['counter_proposal']);
 			unset($properties['responsestatus']);
 			unset($properties['meetingtype']);
 			unset($properties['goid']);
@@ -1084,7 +1050,6 @@
 		function getTaskListProperties()
 		{
 			$properties = $this->getTaskProperties();
-
 			return $properties;
 		}
 
