@@ -347,9 +347,6 @@
 			$this->settings['zarafa']['v1']['contexts']['mail']['outofoffice']['message'] = $props[PR_EC_OUTOFOFFICE_MSG];
 			$this->settings['zarafa']['v1']['contexts']['mail']['outofoffice']['subject'] = $props[PR_EC_OUTOFOFFICE_SUBJECT];
 
-			// FIXME: find a better place to do this
-			$this->settings['zarafa']['v1']['main']['attachments']['max_attachment_size'] = getMaxUploadSize();
-
 			// Save the properties
 			$this->externalSettings = $props;
 		}
