@@ -61,7 +61,7 @@ class State {
 	 */
 	function State($subsystem) {
 		$this->basedir = TMP_PATH . DIRECTORY_SEPARATOR . $this->sessiondir;
-		$this->filename = $this->basedir . DIRECTORY_SEPARATOR . session_id() . "." . $subsystem;
+		$this->filename = $this->basedir . DIRECTORY_SEPARATOR . $GLOBALS["mapisession"]->getSessionID() . "." . $subsystem;
 	}
 
 	/**
