@@ -864,7 +864,7 @@ function sq_fixstyle($body, $pos, $message, $id, $mailbox){
  */
 function sq_body2div($attary, $mailbox, $message, $id){
     $me = 'sq_body2div';
-    $divattary = Array('class' => "'bodyclass'");
+    $divattary = Array('class' => "\"bodyclass\"");
     $bgcolor = '#ffffff';
     $text = '#000000';
     $styledef = '';
@@ -1258,4 +1258,4 @@ function magicHTML($body, $id) {
     return $trusted;
 }
 
-?>
\ No newline at end of file
+?>
