@@ -165,6 +165,7 @@
 					// TODO: use PR_ANR for this restriction instead of PR_DISPLAY_NAME and PR_ACCOUNT
 					$tempRestriction = 	array(RES_OR, 
 										array(
+											// Display name of user from GAB and contacts.
 											array(
 												RES_CONTENT,
 													array(FUZZYLEVEL => FL_SUBSTRING|FL_IGNORECASE,
@@ -172,6 +173,7 @@
 														VALUE => $searchstring
 													)
 												),
+											// fileas value of user from GAB.
 											array(
 												RES_CONTENT,
 													array(FUZZYLEVEL => FL_SUBSTRING|FL_IGNORECASE,
@@ -179,7 +181,39 @@
 														VALUE => $searchstring
 													)
 												),
-										),
+											// smtp_address of user from GAB.
+											array(
+												RES_CONTENT,
+													array(FUZZYLEVEL => FL_SUBSTRING|FL_IGNORECASE,
+														ULPROPTAG => PR_SMTP_ADDRESS,
+														VALUE => $searchstring
+													)
+												),
+											// email_address of user from GAB and contacts.
+											array(
+												RES_CONTENT,
+													array(FUZZYLEVEL => FL_SUBSTRING|FL_IGNORECASE,
+														ULPROPTAG => PR_EMAIL_ADDRESS,
+														VALUE => $searchstring
+													)
+												),
+											// department of user from GAB.
+											array(
+												RES_CONTENT,
+													array(FUZZYLEVEL => FL_SUBSTRING|FL_IGNORECASE,
+														ULPROPTAG => PR_DEPARTMENT_NAME,
+														VALUE => $searchstring
+													)
+												),
+											// fileas of user from Contacts.
+											array(
+												RES_CONTENT,
+													array(FUZZYLEVEL => FL_SUBSTRING|FL_IGNORECASE,
+														ULPROPTAG => PR_ORIGINAL_DISPLAY_NAME,
+														VALUE => $searchstring
+													)
+												)
+										)
 									);
 				} else if(!empty($paginationCharacter)) {
 					// create restriction for alphabet bar
@@ -285,7 +319,7 @@
 						}
 
 						if(!empty($email_address)) {
-							$item['search_key'] = bin2hex(strtoupper($item['address_type']) . ':' . strtoupper($email_address)) . '\0';
+							$item['search_key'] = bin2hex(strtoupper($item['address_type'] . ':' . $email_address)) . '00';
 						}
 					}
 
