@@ -20,6 +20,13 @@
 			$this->plaintext = true;
 
 			parent::ItemModule($id, $data);
+
+			$this->skipCopyProperties = array(
+				$this->properties['goid'],
+				$this->properties['goid2'],
+				$this->properties['request_sent'],
+				PR_OWNER_APPT_ID
+			);
 		}
 		
 		function open($store, $entryid, $action)
@@ -214,11 +221,9 @@
 				$GLOBALS['bus']->addData($this->getResponseData());
 			}else{
 				$this->sendFeedback(false, array(
-						'error' => array(
-							'type' => ERROR_ZARAFA,
-							'info' => array(
-								'display_message' => $errorMsg,
-							)
+						'type' => ERROR_ZARAFA,
+						'info' => array(
+							'display_message' => $errorMsg,
 						)
 					));
 			}
