@@ -16,11 +16,21 @@
 		function HierarchyModule($id, $data)
 		{
 			$this->properties = $GLOBALS["properties"]->getFolderProperties();
+			$this->list_properties = $GLOBALS["properties"]->getFolderListProperties();
 
-			parent::Module($id, $data, array(OBJECT_SAVE, OBJECT_DELETE, TABLE_SAVE, TABLE_DELETE));
+			parent::Module($id, $data);
+		}
 
-			// register for request start/end
-			$GLOBALS["bus"]->register($this, REQUEST_ENTRYID, array(REQUEST_START,REQUEST_END));
+		/**
+		 * Creates the notifiers for this module,
+		 * and register them to the Bus.
+		 */
+		function createNotifiers()
+		{
+			$entryid = $this->getEntryID();
+			$GLOBALS["bus"]->registerNotifier('hierarchynotifier', $entryid, true);
+			$GLOBALS["bus"]->registerNotifier('hierarchynotifier', REQUEST_ENTRYID);
+			$GLOBALS["bus"]->registerNotifier('newmailnotifier', REQUEST_ENTRYID);
 		}
 		
 		/**
@@ -30,20 +40,14 @@
 		 */
 		function getEntryID()
 		{
-			$entryid = array();
+			$entryids = array();
 			$storelist = $GLOBALS["mapisession"]->getAllMessageStores();
 
-			foreach($storelist as $store)
-			{
-				/**
-				 * register notification events to IPM_SUBTREE folder of every store
-				 * because we are not doing anything above folder IPM_SUBTREE
-				 */
-				$store_props = mapi_getprops($store, array(PR_IPM_SUBTREE_ENTRYID));
-				array_push($entryid, bin2hex($store_props[PR_IPM_SUBTREE_ENTRYID]));
+			foreach ($storelist as $entryid => $store) {
+				$entryids[] = bin2hex($entryid);
 			}
 			
-			return $entryid;
+			return $entryids;
 		}
 		
 		/**
@@ -183,7 +187,7 @@
 									// We're closing a Shared folder, check if we still have other
 									// folders for the same user opened, if not we can safely close
 									// the usrstore.
-									$stores = $GLOBALS["settings"]->get("zarafa/v1/contexts/hierarchy/shared_stores/" . $action["user_name"]);
+									$stores = $GLOBALS["settings"]->get("zarafa/v1/contexts/hierarchy/shared_stores/" . strtolower($action["user_name"]));
 									if (!isset($stores) || empty($stores) || (count($stores) == 1 && isset($stores[$action["folder_type"]]))) {
 										$entryid = $GLOBALS["mapisession"]->removeUserStore($action["user_name"]);
 									} else {
@@ -209,7 +213,7 @@
 								$store = $GLOBALS["mapisession"]->addUserStore($username);
 								if ($store) {
 									$options = array( $username => array( $action["folder_type"] => $action ));
-									$data = $GLOBALS["operations"]->getHierarchyList($this->properties, HIERARCHY_GET_ONE, $store, $options);
+									$data = $GLOBALS["operations"]->getHierarchyList($this->list_properties, HIERARCHY_GET_ONE, $store, $options);
 
 									// Check if we have obtained a store, and if that store has read access
 									if (!empty($data["item"][0]["folders"]["item"]) &&
@@ -231,7 +235,7 @@
 								$this->handleUnknownActionType($actionType);
 						}
 					} catch (MAPIException $e) {
-						$this->processException($e, $actionType, $entryid, $action);
+						$this->processException($e, $actionType, $store, $parententryid, $entryid, $action);
 					}
 				}
 			}
@@ -244,10 +248,12 @@
 		 * 
 		 * @param object $e Exception object
 		 * @param string $actionType the action type, sent by the client
+		 * @param MAPIobject $store Store object of the folder.
+		 * @param string $parententryid parent entryid of the message.
 		 * @param string $entryid entryid of the folder.
 		 * @param array $action the action data, sent by the client
 		 */
-		function handleException(&$e, $actionType=null, $entryid=null, $action=null)
+		function handleException(&$e, $actionType = null, $store = null, $parententryid = null, $entryid = null, $action = null)
 		{
 			if(is_null($e->displayMessage)) {
 				switch($actionType) {
@@ -355,7 +361,7 @@
 				}
 			}
 
-			parent::handleException($e, $actionType, $entryid, $action);
+			parent::handleException($e, $actionType, $store, $parententryid, $entryid, $action);
 		}
 		
 		/**
@@ -363,7 +369,7 @@
 		 */
 		function hierarchyList()
 		{
-			$data = $GLOBALS["operations"]->getHierarchyList($this->properties);
+			$data = $GLOBALS["operations"]->getHierarchyList($this->list_properties);
 
 			$this->addActionData("list", $data);
 			$GLOBALS["bus"]->addData($this->getResponseData());
@@ -378,7 +384,7 @@
 		function addFolderToResponseData($store, $entryid, $actionType)
 		{
 			$folder = mapi_msgstore_openentry($store, $entryid);
-			$folderProps = mapi_folder_getprops($folder, $this->properties);
+			$folderProps = mapi_folder_getprops($folder, $this->list_properties);
 
 			$data = $GLOBALS["operations"]->setFolder($folderProps);
 			$this->addActionData($actionType, $data);
@@ -417,7 +423,7 @@
 		*/
 		function getFolderProps($store, $folder)
 		{
-			$data = $GLOBALS["operations"]->getProps($store, $folder, $GLOBALS["properties"]->getFolderProperties());
+			$data = $GLOBALS["operations"]->getProps($store, $folder, $this->properties);
 
 			// adding container_class if missing
 			if (!isset($data["props"]["container_class"])){
@@ -463,14 +469,22 @@
 		 */
 		function getFolderSize($store, $folder, $pathname, &$subfolders, $hidden = false)
 		{
+			$columns = array(PR_ENTRYID, PR_PARENT_ENTRYID, PR_STORE_ENTRYID, PR_OBJECT_TYPE, PR_DISPLAY_NAME, PR_ATTR_HIDDEN);
 			$size = $GLOBALS["operations"]->calcFolderMessageSize($folder, false);
 			$total_size = $size;
+			$batchcount = 50;
 
 			$table = mapi_folder_gethierarchytable($folder, MAPI_DEFERRED_ERRORS);
 
-			$count = mapi_table_getrowcount($table);
-			for ($i = 0; $i < $count; $i += 50) {
-				$rows = mapi_table_queryrows($table, array(PR_ENTRYID, PR_PARENT_ENTRYID, PR_STORE_ENTRYID, PR_OBJECT_TYPE, PR_DISPLAY_NAME, PR_ATTR_HIDDEN), $i, 50);
+			// Check if mapi_table_setcolumns exists (ZCP-7.1.1 and above), if so
+			// then call mapi_table_setcolumns now, and set $columns to null.
+			if (function_exists('mapi_table_setcolumns')) {
+				mapi_table_setcolumns($table, $columns);
+				$columns = null;
+			}
+
+			do {
+				$rows = mapi_table_queryrows($table, $columns, 0, $batchcount);
 				foreach ($rows as $row) {
 					$subfolder = mapi_msgstore_openentry($store, $row[PR_ENTRYID]);
 					$subpath = (!empty($pathname) ? ($pathname . '\\') : '') . $row[PR_DISPLAY_NAME];
@@ -499,7 +513,10 @@
 
 					$total_size += $info["total_size"];
 				}
-			}
+
+			// When the server returned a different number of rows then was requested,
+			// we have reached the end of the table and we should exit the loop.
+			} while (count($rows) == $batchcount);
 
 			return array( "size" => $size, "total_size" => $total_size );
 		}
@@ -566,13 +583,12 @@
 			$storeProps = mapi_getprops($store, array(PR_IPM_SUBTREE_ENTRYID));
 
 			// check if the folder is the default calendar, if so we also need to set the same permissions on the freebusy folder
-			$inbox = mapi_msgstore_getreceivefolder($store);
-			// public store can not have inbox folder
-			if(isset($inbox) && $inbox) {
-				$inboxProps = mapi_getprops($inbox, array(PR_IPM_APPOINTMENT_ENTRYID, PR_FREEBUSY_ENTRYIDS));
-				if ($folderProps[PR_ENTRYID] == $inboxProps[PR_IPM_APPOINTMENT_ENTRYID]){
-					if(isset($inboxProps[PR_FREEBUSY_ENTRYIDS]) && isset($inboxProps[PR_FREEBUSY_ENTRYIDS][3])){
-						$freebusy = mapi_msgstore_openentry($store, $inboxProps[PR_FREEBUSY_ENTRYIDS][3]);
+			$root = mapi_msgstore_openentry($store, null);
+			if($root) {
+				$rootProps = mapi_getprops($root, array(PR_IPM_APPOINTMENT_ENTRYID, PR_FREEBUSY_ENTRYIDS));
+				if ($folderProps[PR_ENTRYID] == $rootProps[PR_IPM_APPOINTMENT_ENTRYID]){
+					if(isset($rootProps[PR_FREEBUSY_ENTRYIDS]) && isset($rootProps[PR_FREEBUSY_ENTRYIDS][3])){
+						$freebusy = mapi_msgstore_openentry($store, $rootProps[PR_FREEBUSY_ENTRYIDS][3]);
 					}
 				}
 			}
@@ -722,13 +738,16 @@
 			$props = array();
 			$result = $GLOBALS["operations"]->deleteFolder($store, $parententryid, $entryid, $props);
 
+			// Indicate if the delete succeedded
+			$this->sendFeedback($result);
+
 			if(isset($props[PR_ENTRYID])) {
 				if($result) {
 					$GLOBALS["bus"]->notify(bin2hex($parententryid), OBJECT_SAVE, $props);
 					
 					$props = array();
 					$props[PR_PARENT_ENTRYID] = $parententryid;
-					
+
 					$storeprops = mapi_getprops($store, array(PR_ENTRYID, PR_IPM_WASTEBASKET_ENTRYID));
 					$props[PR_STORE_ENTRYID] = $storeprops[PR_ENTRYID];
 					$GLOBALS["bus"]->notify(bin2hex($parententryid), OBJECT_SAVE, $props);
@@ -811,19 +830,7 @@
 					} catch (MAPIException $e) {
 						if($e->getCode() == MAPI_E_INVALID_ENTRYID) {
 							// Entryid of the folder might be change after move, so send delete notification for folder.
-							$data = array();
-							$data["folderdelete"] = 1;
-							$data["entryid"] = bin2hex($props[PR_ENTRYID]);
-							$data["store_entryid"] = bin2hex($props[PR_STORE_ENTRYID]);
-							$data["props"] = array();
-							$data["props"]["parent_entryid"] = bin2hex($parententryid);
-							$data["props"]["store_entryid"] = bin2hex($props[PR_STORE_ENTRYID]);
-
-							$this->addNotificationActionData("folders", array( "item" => $data));
-							$GLOBALS["bus"]->addData($this->getNotificationResponseData());
-
-							// We have to clear the response data to prevent that this is submitted to the bus again, when the hierarchymodule is called.
-							$this->responseNotificationData = array();
+							$GLOBALS["bus"]->notify(bin2hex($folderProps[PR_ENTRYID]), OBJECT_DELETE, $props);
 						}
 					}
 
@@ -831,11 +838,13 @@
 					$sourcefolder = mapi_msgstore_openentry($store, $parententryid);
 					$folderProps = mapi_folder_getprops($sourcefolder, array(PR_ENTRYID, PR_STORE_ENTRYID));
 					$GLOBALS["bus"]->notify(bin2hex($folderProps[PR_ENTRYID]), OBJECT_SAVE, $folderProps);
+				} else {
+					$this->sendFeedback(true);
 				}
 
 				// Update subfolders of copy/move folder
 				$folder = mapi_msgstore_openentry($deststore, $destfolderentryid);
-				$hierarchyTable = mapi_folder_gethierarchytable($folder, MAPI_DEFERRED_ERRORS);
+				$hierarchyTable = mapi_folder_gethierarchytable($folder, CONVENIENT_DEPTH | MAPI_DEFERRED_ERRORS);
 				mapi_table_sort($hierarchyTable, array(PR_DISPLAY_NAME => TABLE_SORT_ASCEND), TBL_BATCH);
 
 				/**
@@ -877,6 +886,12 @@
 				$folder = mapi_msgstore_openentry($deststore, $destfolderentryid);
 				$folderProps = mapi_folder_getprops($folder, array(PR_ENTRYID, PR_STORE_ENTRYID));
 				$GLOBALS["bus"]->notify(bin2hex($folderProps[PR_ENTRYID]), OBJECT_SAVE, $folderProps);
+			} else {
+				if ($moveFolder) {
+					$this->sendFeedback(false, _('Could not move folder'));
+				} else {
+					$this->sendFeedback(false, _('Could not copy folder'));
+				}
 			}
 		}
 		
@@ -898,175 +913,7 @@
 				$GLOBALS["bus"]->addData($this->getResponseData());
 
 				$GLOBALS["bus"]->notify(bin2hex($props[PR_ENTRYID]), OBJECT_SAVE, $props);
-				$GLOBALS["bus"]->notify(bin2hex($props[PR_ENTRYID]), TABLE_SAVE, $props);
 			}
 		}
-		
-		/**
-		 * If an event elsewhere has occurred, it enters in this methode. This method
-		 * executes one ore more actions, depends on the event.
-		 * @param int $event Event.
-		 * @param string $entryid Entryid.
-		 * @param array $data array of data.
-		 */
-		function update($event, $entryid, $props)
-		{
-			switch($event)
-			{
-				case OBJECT_SAVE:
-				case TABLE_SAVE:
-				case TABLE_DELETE:
-					$data = array();
-
-					// If no PR_ENTRYID is given, we will settle with PR_PARENT_ENTRYID
-					$folderEntryid = false;
-					if(isset($props[PR_PARENT_ENTRYID])) {
-						$folderEntryid = $props[PR_PARENT_ENTRYID];
-					} else if(isset($props[PR_ENTRYID])) {
-						$folderEntryid = $props[PR_ENTRYID];
-					}
-
-					if(isset($props[PR_STORE_ENTRYID]) && $folderEntryid && !$this->isFolderInResponseData(bin2hex($folderEntryid))) {
-						$store = $GLOBALS["mapisession"]->openMessageStore($props[PR_STORE_ENTRYID]);
-						
-						$folder = mapi_msgstore_openentry($store, $folderEntryid);
-						if($folder) {
-							$folderProps = mapi_folder_getprops($folder, $this->properties);
-
-							// If this folder belongs to Favorites folder,then change PARENT_ENTRYID manually.
-							if (substr(bin2hex($folderProps[PR_ENTRYID]), 0, 8) == "00000001") {
-								$storeProps = mapi_getprops($store, array(PR_IPM_FAVORITES_ENTRYID));
-
-								if (isset($storeProps[PR_IPM_FAVORITES_ENTRYID])) {
-									$favFolder = mapi_msgstore_openentry($store, $storeProps[PR_IPM_FAVORITES_ENTRYID]);
-									$favHierarchyTable = mapi_folder_gethierarchytable($favFolder, MAPI_DEFERRED_ERRORS);
-
-									$folders = mapi_table_queryallrows($favHierarchyTable, array(PR_DISPLAY_NAME, PR_STORE_ENTRYID), array(RES_PROPERTY,
-																											array(  RELOP => RELOP_EQ,
-																													ULPROPTAG => PR_ENTRYID,
-																													VALUE => array (PR_ENTRYID => $folderProps[PR_ENTRYID])
-																											)
-																										)
-																	);
-
-									// Update folderProps to properties of folder which is under 'FAVORITES'
-									$folderProps[PR_DISPLAY_NAME] = $folders[0][PR_DISPLAY_NAME];
-									$folderProps[PR_PARENT_ENTRYID] = $storeProps[PR_IPM_FAVORITES_ENTRYID];
-								}
-							}
-							
-							$data[] = $GLOBALS["operations"]->setFolder($folderProps);
-						}
-
-						$this->addNotificationActionData("folders", array( "item" => $data ));
-						$GLOBALS["bus"]->addData($this->getNotificationResponseData());
-
-						// We have to clear the response data to prevent that this is submitted to the bus again, when the hierarchymodule is called.
-						$this->responseNotificationData = array();
-					}
-
-					break;
-				case OBJECT_DELETE:
-
-					if(isset($props[PR_ENTRYID]) && isset($props[PR_PARENT_ENTRYID]) && !$this->isFolderInResponseData(bin2hex($props[PR_ENTRYID]))) {
-						$data = array();
-						$data["folderdelete"] = 1;
-						$data["entryid"] = bin2hex($props[PR_ENTRYID]);
-						$data["parent_entryid"] = bin2hex($props[PR_PARENT_ENTRYID]);
-						$data["store_entryid"] = bin2hex($props[PR_STORE_ENTRYID]);
-
-						$this->addNotificationActionData("folders", array( "item" => $data ));
-						$GLOBALS["bus"]->addData($this->getNotificationResponseData());
-
-						// We have to clear the response data to prevent that this is submitted to the bus again, when the hierarchymodule is called.
-						$this->responseNotificationData = array();
-					}
-
-					break;
-				case REQUEST_START:
-					$this->reset();
-
-					$inbox = mapi_msgstore_getreceivefolder($GLOBALS["mapisession"]->getDefaultMessageStore());
-
-					$newmail = $this->checkNewMail($inbox);
-					if ($newmail != false) {
-						$inboxProps = mapi_getprops($inbox, array(PR_ENTRYID, PR_STORE_ENTRYID, PR_CONTENT_COUNT, PR_CONTENT_UNREAD, PR_DISPLAY_NAME));
-
-						$data = array(
-							"item" => array(
-								// Currenly only the inbox is checked, but in the future this will be expanded to multiple folders.
-								array(
-									"content_count" => $inboxProps[PR_CONTENT_COUNT],
-									"content_unread" => $inboxProps[PR_CONTENT_UNREAD],
-									// Add store_entryid,entryid of folder and display_name of folder 
-									// to JSON data in order to refresh the list.
-									"store_entryid" => bin2hex($inboxProps[PR_STORE_ENTRYID]),
-									"entryid" => bin2hex($inboxProps[PR_ENTRYID]),
-									"display_name" => $inboxProps[PR_DISPLAY_NAME]
-								)
-							)
-						);
-
-						$this->addNotificationActionData("newmail", $data);
-						$GLOBALS["bus"]->addData($this->getNotificationResponseData());
-
-						// We have to clear the response data to prevent the notify call below to submit this again to the bus, when the hierarchymodule is called.
-						$this->responseNotificationData = array();
-
-						$GLOBALS["bus"]->notify(bin2hex($inboxProps[PR_ENTRYID]), OBJECT_SAVE, $inboxProps);
-					}
-					break;
-				case REQUEST_END:
-					$inbox = mapi_msgstore_getreceivefolder($GLOBALS["mapisession"]->getDefaultMessageStore());
-					// Update unread counter now, so that any changes that we have done ourselved (for example
-					// moving an unread e-mail into the inbox) will not be detected by the newMail detection in
-					// REQUEST_START. We're not interested in the return value though.
-					$this->checkNewMail($inbox);
-
-					//TODO add update for the counters of the quota.
-
-					break;	
-
-			}
-		}
-
-		/**
-		 * Function will check 'inbox' folder for new mails.
-		 * initially we store PR_CONTENT_UNREAD count property of inbox folder to module
-		 * then every time we compare that property with new value and if its changed then
-		 * we can assume that new mail has arrived
-		 * @param MAPIFolder $inbox inbox folder
-		 * @return boolean true if count is change else false.
-		 */
-		function checkNewMail($inbox)
-		{
-			$result = false;
-			
-			$inboxprops = mapi_getprops($inbox, array(PR_CONTENT_UNREAD));
-			
-			if(isset($this->lastUnread)) {
-				if($this->lastUnread < $inboxprops[PR_CONTENT_UNREAD])
-					$result = true;
-			}
-			
-			$this->lastUnread = $inboxprops[PR_CONTENT_UNREAD];
-			return $result;
-		}
-
-		function isFolderInResponseData($folderEntryid)
-		{
-			if (!empty($this->responseData)) {
-				foreach($this->responseData as $actionType => $data) {
-					if (is_array($data) && isset($data["entryid"])) {
-						if ($folderEntryid == $data["entryid"]) {
-							return true;
-						}
-					}
-				}
-			}
-
-			return false;
-		}
-
 	}
 ?>
