@@ -11,6 +11,11 @@
 		 */
 		var $directBookingMeetingRequest;
 
+		/**
+		 * The array of properties which should not be copied during the copy() action.
+		 */
+		var $skipCopyProperties;
+
 		/**
 		 * Constructor
 		 * @param int $id unique id.
@@ -19,6 +24,7 @@
 		function ItemModule($id, $data)
 		{
 			$this->directBookingMeetingRequest = ENABLE_DIRECT_BOOKING;
+			$this->skipCopyProperties = array();
 			parent::Module($id, $data);
 		}
 		
@@ -51,7 +57,7 @@
 									 *   - declineMeetingRequest: attendee has declined mr
 									 */
 									if (isset($action["message_action"]) && isset($action["message_action"]["action_type"])) {
-										switch($action["message_action"]["action_type"])
+										switch($action["message_action"]["action_type"]) 
 										{
 											case "declineMeetingRequest":
 											case "acceptMeetingRequest":
@@ -111,7 +117,7 @@
 												 */
 												$messageProps = mapi_getprops($message, array(PR_ENTRYID, PR_STORE_ENTRYID, PR_PARENT_ENTRYID));
 
-												if($delete){
+												if($delete) {
 													// send TABLE_DELETE event because the message has moved
 													$this->sendFeedback(true);
 													$GLOBALS["bus"]->notify(bin2hex($messageProps[PR_PARENT_ENTRYID]), TABLE_DELETE, $messageProps);
@@ -125,10 +131,6 @@
 
 											break;
 
-											case "read_flag":
-												$this->setReadFlag($store, $parententryid, $entryid, $action);
-												break;
-
 											case "copy":
 											case "move":
 												$this->copy($store, $parententryid, $entryid, $action);
@@ -184,7 +186,6 @@
 									case "declineMeeting":
 										// @FIXME can we somehow merge declineMeeting and declineMeetingRequest sub actions?
 										$message = $GLOBALS["operations"]->openMessage($store, $entryid);
-										// @FIXME: 'basedate' should be moved, maybe either in 'props' or 'message_action'?
 										$basedate = (isset($action['basedate']) && !empty($action['basedate'])) ? $action['basedate'] : false;
 
 										$req = new Meetingrequest($store, $message, $GLOBALS["mapisession"]->getSession(), $this->directBookingMeetingRequest);
@@ -219,9 +220,6 @@
 										break;
 								}
 								break;
-							case "checknames":
-								$this->checkNames($store, $action);
-								break;
 
 							case "resolveConflict":
 								$this->resolveConflict($store, $parententryid, $entryid, $action);
@@ -314,14 +312,6 @@
 						}
 						break;
 
-					case "checknames":
-					case "read_flag":
-						if($e->getCode() == MAPI_E_NO_ACCESS)
-							$e->setDisplayMessage(_("You have insufficient privileges to perform this action."));
-						else
-							$e->setDisplayMessage(_("Could not perform action."));
-						break;
-
 					case "removeFromCalendar":
 						if($e->getCode() == MAPI_E_NO_ACCESS)
 							$e->setDisplayMessage(_("You have insufficient privileges to remove item from the calendar."));
@@ -525,6 +515,7 @@
 		function save($store, $parententryid, $entryid, $action)
 		{
 			$result = false;
+			$resultFlags = false;
 			
 			if(isset($action["props"])) {
 				if(!$store && !$parententryid) {
@@ -539,15 +530,32 @@
 
 				if($store && $parententryid) {
 					$props = Conversion::mapXML2MAPI($this->properties, $action["props"]);
-
-					$messageProps = array(); // returned props
-					$result = $GLOBALS["operations"]->saveMessage($store, $entryid, $parententryid, $props, $messageProps, array(), (isset($action['attachments']) ? $action['attachments'] : array()));
 					
-					if($result) {
+					/*
+					 * Set message flags separately from other properties, because we want to be able
+					 * to mark messages without having write permissions on a store.
+					 * For instance a delegate user should still be able to mark messages as read.
+					 */
+					if(isset($props[PR_MESSAGE_FLAGS])){
+						$resultFlags = $GLOBALS["operations"]->setMessageFlag($store, $entryid, $props[PR_MESSAGE_FLAGS]);
+						unset($props[PR_MESSAGE_FLAGS]);
+					} else {
+						$resultFlags = true;
+					}
+
+					$messageProps = array(); // props returned from saveMessage
+					// Save message if there are any properties left and setting the flags didn't fail
+					if($resultFlags && !empty($props)){
+						$result = $GLOBALS["operations"]->saveMessage($store, $entryid, $parententryid, $props, $messageProps, array(), (isset($action['attachments']) ? $action['attachments'] : array()));
+					}
+
+					if($result && $resultFlags) {
 						$GLOBALS["bus"]->notify(bin2hex($parententryid), TABLE_SAVE, $messageProps);
 
 						$this->addActionData("update", array("item" => Conversion::mapMAPI2XML($this->properties, $messageProps)));
 						$GLOBALS["bus"]->addData($this->getResponseData());
+					} else {
+						$this->sendFeedback(false);
 					}
 				}
 			}
@@ -763,7 +771,7 @@
 				$storeprops = mapi_getprops($store, array(PR_ENTRYID));
 				$props[PR_STORE_ENTRYID] = $storeprops[PR_ENTRYID];
 
-				$result = $GLOBALS["operations"]->copyMessages($store, $parententryid, $dest_store, $dest_folderentryid, $entryids, $moveMessages);
+				$result = $GLOBALS["operations"]->copyMessages($store, $parententryid, $dest_store, $dest_folderentryid, $entryids, $moveMessages ? array() : $this->skipCopyProperties, $moveMessages);
 
 				if($result) {
 					if($moveMessages) {
@@ -793,6 +801,7 @@
 		function getCalendarItemProps($meetingRequestObject)
 		{
 			$calendarItem = $meetingRequestObject->getCorrespondentCalendarItem();
+			$props = array();
 			if ($calendarItem !== false) {
 				$calendarItemProps = mapi_getprops($calendarItem, array(PR_STORE_ENTRYID, PR_PARENT_ENTRYID, PR_ENTRYID, $meetingRequestObject->proptags['updatecounter'], $meetingRequestObject->proptags['goid']));
 
@@ -821,7 +830,7 @@
 				}
 
 				if($meetingRequestObject->isMeetingRequestResponse()) {
-					$data['item']['props']['meeting_updated'] = $meetingRequestObject->isMeetingUpdated(true, $basedate);
+					$props['meeting_updated'] = $meetingRequestObject->isMeetingUpdated($basedate);
 				}
 
 				return $props;
