@@ -31,11 +31,6 @@
 		 */
 		var $searchFolderList;
 
-		/**
-		 * @var hexString entryid of the created search folder
-		 */
-		var $searchFolderEntryId;
-
 		/**
 		 * @var array stores search criteria of previous request
 		 */
@@ -71,13 +66,11 @@
 			
 			$this->searchRestriction = false;
 			$this->searchFolderList = false;
-			$this->searchFolderEntryId = false;
-			$this->searchCriteriaCheck = false;
 			$this->localFreeBusyFolder = false;
 			$this->storeProviderGuid = false;
 
 			$this->sort = array();
-			
+
 			parent::Module($id, $data);
 		}
 		
@@ -106,6 +99,8 @@
 						}
 					} catch (MAPIException $e) {
 						$this->processException($e, $actionType, $store, $parententryid, $entryid, $action);
+					} catch (SearchException $e) {
+						$this->processException($e, $actionType, $store, $parententryid, $entryid, $action);
 					}
 				}
 			}
@@ -252,14 +247,29 @@
 				$entryids = $entryid;
 			}
 
+			$searchFolderEntryId = $this->sessionData['searchFolderEntryId'];
+
 			// check if searchcriteria has changed
-			$restrictionCheck = md5(serialize($this->searchRestriction) . serialize($entryids) . $subfolder_flag);
+			$restrictionCheck = md5(serialize($this->searchRestriction) . $searchFolderEntryId . $subfolder_flag);
 
 			// check if there is need to set searchcriteria again
-			if($restrictionCheck != $this->searchCriteriaCheck) {
+			if(!isset($this->sessionData['searchCriteriaCheck']) || $restrictionCheck != $this->sessionData['searchCriteriaCheck']) {
+				if (!empty($this->sessionData['searchOriginalEntryids'])) {
+					// get entryids of original folders, and use it to set new search criteria
+					$entryids = Array();
+					for($index = 0; $index < count($this->sessionData['searchOriginalEntryids']); $index++) {
+						$entryids[] = hex2bin($this->sessionData['searchOriginalEntryids'][$index]);
+					}
+				} else {
+					// store entryids of original folders, so that can be used for re-setting the search criteria if needed
+					$this->sessionData['searchOriginalEntryids'] = Array();
+					for($index = 0, $len = count($entryids); $index < $len; $index++) {
+						$this->sessionData['searchOriginalEntryids'][] = bin2hex($entryids[$index]);
+					}
+				}
+
 				mapi_folder_setsearchcriteria($searchFolder, $this->searchRestriction, $entryids, $subfolder_flag);
-				// @FIXME currently not working as we have to retrieve/store this information in sessionData
-				$this->searchCriteriaCheck = $restrictionCheck;
+				$this->sessionData['searchCriteriaCheck'] = $restrictionCheck;
 			}
 
 			unset($action["restriction"]);
@@ -289,7 +299,7 @@
 			}
 
 			// Get the table and merge the arrays
-			$table = $GLOBALS["operations"]->getTable($store, hex2bin($this->searchFolderEntryId), $this->properties, $this->sort, $this->start);
+			$table = $GLOBALS["operations"]->getTable($store, hex2bin($searchFolderEntryId), $this->properties, $this->sort, $this->start);
 			$data = array_merge($data, $table);
 
 			// remember which entryid's are send to the client
@@ -303,12 +313,12 @@
 			if(!isset($this->sessionData['searchResults'])) {
 				$this->sessionData['searchResults'] = array();
 			}
-			$this->sessionData['searchResults'][$this->searchFolderEntryId] = $searchResults;
+			$this->sessionData['searchResults'][$searchFolderEntryId] = $searchResults;
 
 			$result = mapi_folder_getsearchcriteria($searchFolder);
 
 			$data["search_meta"] = array();
-			$data["search_meta"]["searchfolder_entryid"] = $this->searchFolderEntryId;
+			$data["search_meta"]["searchfolder_entryid"] = $searchFolderEntryId;
 			$data["search_meta"]["searchstate"] = $result["searchstate"];
 			$data["search_meta"]["results"] = count($searchResults);
 
@@ -436,13 +446,21 @@
 		 */
 		function stopSearch($store, $entryid, $action)
 		{
-			if(!isset($entryid) || !$entryid) {
-				// if no entryid is present then we can't do anything here
+			// if no entryid is present in the request then get the search folder entryid from session data
+			$entryid = !empty($entryid) ? $entryid : hex2bin($this->sessionData['searchFolderEntryId']);
+
+			if(empty($entryid)) {
+				// still no entryid? sorry i can't help you anymore
+				$this->addActionData("stopsearch", array( 'success' => false ));
+				$GLOBALS["bus"]->addData($this->getResponseData());
 				return;
 			}
 
 			// remove search result entryids stored in session
 			unset($this->sessionData['searchResults'][bin2hex($entryid)]);
+			unset($this->sessionData['searchCriteriaCheck']);
+			unset($this->sessionData['searchFolderEntryId']);
+			unset($this->sessionData['searchOriginalEntryids']);
 
 			$searchFolder = mapi_msgstore_openentry($store, $entryid);
 			$searchResult = mapi_folder_getsearchcriteria($searchFolder);
@@ -456,9 +474,7 @@
 			 * when stopping search process, we have to remove search folder also,
 			 * so next search request with same restriction will not get uncompleted results
 			 */
-			if(method_exists($this, "deleteSearchFolder")) {
-				$this->deleteSearchFolder($store, $entryid, $action);
-			}
+			$this->deleteSearchFolder($store, $entryid, $action);
 
 			// send success message to client
 			$this->addActionData("stopsearch", array( 'success' => true ));
@@ -504,10 +520,6 @@
 					mapi_folder_deletefolder($finderFolder, $folders[0][PR_ENTRYID]);
 				}
 
-				// reset variables
-				$this->searchFolderEntryId = false;
-				$this->searchCriteriaCheck = false;
-
 				return true;
 			}
 
@@ -523,11 +535,17 @@
 		 */
 		function createSearchFolder($store, $openIfExists = true)
 		{
-			if($this->searchFolderEntryId && $openIfExists) {
-				$searchFolder = mapi_msgstore_openentry($store, hex2bin($this->searchFolderEntryId));
-				if($searchFolder !== false && mapi_last_hresult() == NOERROR) {
-					// search folder exists, don't create new search folder
-					return $searchFolder;
+			if(isset($this->sessionData['searchFolderEntryId']) && $openIfExists) {
+				try {
+					$searchFolder = mapi_msgstore_openentry($store, hex2bin($this->sessionData['searchFolderEntryId']));
+
+					if($searchFolder !== false) {
+						// search folder exists, don't create new search folder
+						return $searchFolder;
+					}
+				} catch (MAPIException $e) {
+					// ignore error and continue creation of search folder
+					unset($this->sessionData['searchFolderEntryId']);
 				}
 			}
 
@@ -545,7 +563,10 @@
 				$searchFolder = mapi_folder_createfolder($searchFolderRoot, $folderName, null, 0, FOLDER_SEARCH);
 
 				$props = mapi_getprops($searchFolder, array(PR_ENTRYID));
-				$this->searchFolderEntryId = bin2hex($props[PR_ENTRYID]);
+				$this->sessionData['searchFolderEntryId'] = bin2hex($props[PR_ENTRYID]);
+
+				// we have created new search folder so search criteria check should be removed
+				unset($this->sessionData['searchCriteriaCheck']);
 
 				return $searchFolder;
 			} catch (MAPIException $e) {
@@ -624,7 +645,8 @@
 				}
 
 				if(!empty($action["restriction"]["search"])) {
-					$this->searchRestriction = Conversion::json2restriction($action["restriction"]["search"]);
+					$props = array_merge($this->properties, array('body' => PR_BODY));
+					$this->searchRestriction = Conversion::json2restriction($props, $action["restriction"]["search"]);
 				} else {
 					$this->searchRestriction = false;
 				}
@@ -702,51 +724,6 @@
 			}
 		}
 
-		/**
-		 * Function which create a task from a mail by copying its subject and body.
-		 * @param object $store MAPI Message Store Object
-		 * @param string $entryid entryid of the item
-		 * @return boolean true on success or false on failure		 		 
-		 */
-		function createTaskFromMail($store, $entryid)
-		{
-			if($store && $entryid) {
-				$root = mapi_msgstore_openentry($store, null);
-				$rootprops = mapi_getprops($root, Array(PR_IPM_TASK_ENTRYID, PR_STORE_ENTRYID));
-				$parententryid = bin2hex($rootprops[PR_IPM_TASK_ENTRYID]);
-
-				// open the userstore to fetch the name of the owner of the store.
-				$userstore = mapi_openmsgstore($GLOBALS["mapisession"]->getSession(),$rootprops[PR_STORE_ENTRYID]);
-				$userstoreprops = mapi_getprops($userstore, Array(PR_MAILBOX_OWNER_NAME));
-
-				// open the mail to fetch the subject, body and importance.
-				$message = mapi_msgstore_openentry($store, $entryid);
-				$msgprops= mapi_getprops($message, Array(PR_SUBJECT, PR_BODY, PR_IMPORTANCE));
-
-				$props = array ('message_class' => 'IPM.Task',
-								'icon_index' => '1280',
-								'subject' => $msgprops[PR_SUBJECT],
-								'importance' => $msgprops[PR_IMPORTANCE],
-								'body' => $msgprops[PR_BODY],
-								'owner' => $userstoreprops[PR_MAILBOX_OWNER_NAME],
-								'complete' => "false",
-								);
-				
-				$messageProps = array(); // returned props
-				$result = $GLOBALS["operations"]->saveMessage($store, hex2bin($parententryid), false, Conversion::mapXML2MAPI($GLOBALS["properties"]->getTaskProperties(), $props), $messageProps);
-				
-				$data = array();
-				$data["success"] = $result;
-
-				$this->addActionData("task_created", $data);
-				$GLOBALS["bus"]->addData($this->getResponseData());
-
-				if($result) {
-					$GLOBALS["bus"]->notify(bin2hex($parententryid), TABLE_SAVE, $messageProps);
-				}
-			}
-		}
-
 		/**
 		 * Function which gets the delegation details from localfreebusy folder to use in
 		 * processPrivateItems function.
