@@ -152,12 +152,6 @@
 							$e->setDisplayMessage(_("Could not cancel Meeting Request invitations."));
 						break;
 
-					case "read_flag":
-						if($e->getCode() == MAPI_E_NO_ACCESS)
-							$e->setDisplayMessage(_("You have insufficient privileges to perform this action."));
-						else
-							$e->setDisplayMessage(_("Could not perform action."));
-						break;
 					case "updatesearch":
 					case "stopsearch":
 					case "search":
@@ -239,26 +233,22 @@
 				if($exception instanceof MAPIException) {
 					$exception->setHandled();
 					return array(
-						"error" => array(
-							"type" => ERROR_MAPI,
-							"info" => array(
-								"hresult" => $exception->getCode(),
-								"hresult_name" => get_mapi_error_name($exception->getCode()),
-								"file" => $exception->getFileLine(),
-								"display_message" => $exception->getDisplayMessage()
-							)
+						"type" => ERROR_MAPI,
+						"info" => array(
+							"hresult" => $exception->getCode(),
+							"hresult_name" => get_mapi_error_name($exception->getCode()),
+							"file" => $exception->getFileLine(),
+							"display_message" => $exception->getDisplayMessage()
 						)
 					);
 				} else if($exception instanceof ZarafaException) {
 					$exception->setHandled();
 					return array(
-						"error" => array(
-							"type" => ERROR_ZARAFA,
-							"info" => array(
-								"file" => $exception->getFileLine(),
-								"display_message" => $exception->getDisplayMessage(),
-								"original_message" => $exception->getMessage()
-							)
+						"type" => ERROR_ZARAFA,
+						"info" => array(
+							"file" => $exception->getFileLine(),
+							"display_message" => $exception->getDisplayMessage(),
+							"original_message" => $exception->getMessage()
 						)
 					);
 				}
