@@ -24,7 +24,7 @@
 					try {
 						switch($actionType)
 						{
-							case "checknames":
+							case 'checknames':
 								$this->checkNames($action);
 								break;
 							default:
@@ -47,12 +47,12 @@
 		 */
 		function checkNames($action)
 		{
-			if(isset($action["resolverequests"])) {
+			if(isset($action['resolverequests'])) {
 				$data = array();
-				$excludeLocalContacts = !empty($action["exclude_local_contacts"]) ? $action["exclude_local_contacts"] : false;
-				$excludeGABGroups = !empty($action["exclude_gab_groups"]) ? $action["exclude_gab_groups"] : false;
+				$excludeLocalContacts = !empty($action['exclude_local_contacts']) ? $action['exclude_local_contacts'] : false;
+				$excludeGABGroups = !empty($action['exclude_gab_groups']) ? $action['exclude_gab_groups'] : false;
 
-				$resolveRequest = $action["resolverequests"];
+				$resolveRequest = $action['resolverequests'];
 				if(!is_array($resolveRequest)) {
 					$resolveRequest = array($resolveRequest);
 				}
@@ -60,7 +60,7 @@
 				// open addressbook
 				// When local contacts need to be excluded we have pass true as the first argument 
 				// so we will not have any Contact Providers set on the Addressbook resource.
-				$ab = $GLOBALS["mapisession"]->getAddressbook($excludeLocalContacts);
+				$ab = $GLOBALS['mapisession']->getAddressbook($excludeLocalContacts);
 
 				$ab_entryid = mapi_ab_getdefaultdir($ab);
 				$ab_dir = mapi_ab_openentry($ab, $ab_entryid);
@@ -68,21 +68,21 @@
 
 				// check names
 				foreach($resolveRequest as $query) {
-					if (is_array($query) && isset($query["id"]) && isset($query["resolvename"]) && !empty($query["resolvename"])) {
+					if (is_array($query) && isset($query['id'])) {
 						$responseEntry = Array();
-						$responseEntry['id'] = $query["id"];
-						$resolvename = $query["resolvename"];
+						$responseEntry['id'] = $query['id'];
 
-						// first resolve in the global addressbook
-						$responseEntry['result'] = $this->searchAddressBook($ab, $ab_dir, $resolvename, $excludeGABGroups);
-
-						$resolveResponse[] = $responseEntry;
+						if (!empty($query['display_name']) || !empty($query['email_address'])) {
+							$responseEntry['result'] = $this->searchAddressBook($ab, $ab_dir, $query, $excludeGABGroups);
+							$resolveResponse[] = $responseEntry;
+						}
 					}
 				}
 
 				$data['resolveresponse'] = $resolveResponse;
-				$this->addActionData("checknames", $data);
-				$GLOBALS["bus"]->addData($this->getResponseData());
+
+				$this->addActionData('checknames', $data);
+				$GLOBALS['bus']->addData($this->getResponseData());
 			}
 		}
 
@@ -97,9 +97,18 @@
 		 */
 		function searchAddressBook($ab, $ab_dir, $query, $excludeGABGroups)
 		{
+			// Prefer resolving the email_address. This allows the user
+			// to resolve recipients with a display name that matches a ZARAFA
+			// user with an alternative (external) email address.
+			$searchstr = empty($query['email_address']) ? $query['display_name'] : $query['email_address'];
+			// If the address_type is 'ZARAFA' then we are resolving something which must be found in
+			// the GAB as an exact match. So add the flag EMS_AB_ADDRESS_LOOKUP to ensure we will not
+			// get multiple results when multiple items have a partial match.
+			$flags = $query['address_type'] === 'ZARAFA' ? EMS_AB_ADDRESS_LOOKUP : 0;
+
 			try {
 				// First, try an addressbook lookup
-				$rows = mapi_ab_resolvename($ab, array ( array(PR_DISPLAY_NAME => $query) ) , 0 );
+				$rows = mapi_ab_resolvename($ab, array ( array(PR_DISPLAY_NAME => $searchstr) ) , $flags);
 			} catch (MAPIException $e) {
 				if ($e->getCode() == MAPI_E_AMBIGUOUS_RECIP) {
 					// Ambiguous, show possiblities:
@@ -114,14 +123,14 @@
 									array(
 										FUZZYLEVEL => FL_PREFIX | FL_IGNORECASE,
 										ULPROPTAG => PR_DISPLAY_NAME,
-										VALUE => $query
+										VALUE => $searchstr
 									)
 								),
 								array(RES_CONTENT,
 									array(
 										FUZZYLEVEL => FL_PREFIX | FL_IGNORECASE,
 										ULPROPTAG => PR_ACCOUNT,
-										VALUE => $query
+										VALUE => $searchstr
 									)
 								),
 							), // RES_OR
@@ -164,27 +173,32 @@
 					mapi_table_restrict($table, $restriction, TBL_BATCH);
 					mapi_table_sort($table, array(PR_DISPLAY_NAME => TABLE_SORT_ASCEND), TBL_BATCH);
 
-					$rows = mapi_table_queryallrows($table, array(PR_ACCOUNT, PR_DISPLAY_NAME, PR_ENTRYID, PR_SEARCH_KEY, PR_OBJECT_TYPE, PR_SMTP_ADDRESS, PR_DISPLAY_TYPE_EX, PR_EMAIL_ADDRESS, PR_OBJECT_TYPE, PR_DISPLAY_TYPE));
+					$rows = mapi_table_queryallrows($table, array(PR_ACCOUNT, PR_ADDRTYPE, PR_DISPLAY_NAME, PR_ENTRYID, PR_SEARCH_KEY, PR_OBJECT_TYPE, PR_SMTP_ADDRESS, PR_DISPLAY_TYPE_EX, PR_EMAIL_ADDRESS, PR_OBJECT_TYPE, PR_DISPLAY_TYPE));
 
-					$rows = array_merge($rows, $this->getAmbigiousContactResolveResults($ab, $query, $excludeGABGroups));
+					$rows = array_merge($rows, $this->getAmbigiousContactResolveResults($ab, $searchstr, $excludeGABGroups));
 				} else if ($e->getCode() == MAPI_E_NOT_FOUND) {
 					$rows = array();
+					// If we still can't find anything, and we were searching for a SMTP user
+					// we can generate a oneoff entry which contains the information of the user.
+					if ($query['address_type'] === 'SMTP' && !empty($query['email_address'])) {
+						$rows[] = array(
+							PR_ENTRYID => mapi_createoneoff($query['display_name'], 'SMTP', $query['email_address'])
+						);
+					}
 				} else {
 					// all other errors should be propagated to higher level exception handlers
 					throw $e;
 				}
 			}
 
-			if (!$rows) {
-				$items = array(); // Nothing found
-			} else {
-				$items = array();
+			$items = array();
+			if ($rows) {
 				foreach($rows as $user_data) {
 					$item = array();
 
 					if (!isset($user_data[PR_ACCOUNT])) {
 						$abitem = mapi_ab_openentry($ab, $user_data[PR_ENTRYID]);
-						$user_data = mapi_getprops($abitem, array(PR_ACCOUNT, PR_DISPLAY_NAME, PR_DISPLAY_TYPE_EX, PR_ENTRYID, PR_SEARCH_KEY, PR_EMAIL_ADDRESS, PR_SMTP_ADDRESS, PR_OBJECT_TYPE, PR_DISPLAY_TYPE));
+						$user_data = mapi_getprops($abitem, array(PR_ACCOUNT, PR_ADDRTYPE, PR_DISPLAY_NAME, PR_DISPLAY_TYPE_EX, PR_ENTRYID, PR_SEARCH_KEY, PR_EMAIL_ADDRESS, PR_SMTP_ADDRESS, PR_OBJECT_TYPE, PR_DISPLAY_TYPE));
 					}
 
 					if($excludeGABGroups && $user_data[PR_OBJECT_TYPE] === MAPI_DISTLIST) {
@@ -193,32 +207,35 @@
 					}
 
 					$item = array();
-					$item["object_type"] = $user_data[PR_OBJECT_TYPE];
-					$item["entryid"] = bin2hex($user_data[PR_ENTRYID]);
-					$item["search_key"] = bin2hex($user_data[PR_SEARCH_KEY]);
-					$item["display_name"] = $user_data[PR_DISPLAY_NAME];
-					$item["display_type"] = $user_data[PR_DISPLAY_TYPE];
+					$item['object_type'] = isset($user_data[PR_OBJECT_TYPE]) ? $user_data[PR_OBJECT_TYPE] : MAPI_MAILUSER;
+					$item['entryid'] = isset($user_data[PR_ENTRYID]) ? bin2hex($user_data[PR_ENTRYID]) : '';
+					$item['display_name'] = isset($user_data[PR_DISPLAY_NAME]) ? $user_data[PR_DISPLAY_NAME] : '';
+					$item['display_type'] = isset($user_data[PR_DISPLAY_TYPE]) ? $user_data[PR_DISPLAY_TYPE] : DT_MAILUSER;
 
-
-					$entryIdUtil = new EntryId();
 					// Test whether the GUID in the entryid is from the Contact Provider
-					if($entryIdUtil->hasContactProviderGUID($item["entryid"])){
+					if($GLOBALS['entryid']->hasContactProviderGUID($item['entryid'])){
 						// The properties for a Distribution List differs from the other objects
 						if($item['object_type'] == MAPI_DISTLIST) {
-							$item["address_type"] = 'MAPIPDL';
+							$item['address_type'] = 'MAPIPDL';
 							// The email_address is empty for DistList, using display name for resolving
-							$item["email_address"] = $user_data[PR_DISPLAY_NAME];
+							$item['email_address'] = $item['display_name'];
 						}else{
-							$item["address_type"] = 'ZARAFA';
-							$item["email_address"] = $user_data[PR_EMAIL_ADDRESS];
+							$item['address_type'] = 'ZARAFA';
+							$item['email_address'] = isset($user_data[PR_EMAIL_ADDRESS]) ? $user_data[PR_EMAIL_ADDRESS] : '';
 						}
 					// It can be considered a GAB entry
-					}else{
-						$item["user_name"] = $user_data[PR_ACCOUNT];
-						$item["display_type_ex"] = $user_data[PR_DISPLAY_TYPE_EX];
-						$item["smtp_address"] =$user_data[PR_SMTP_ADDRESS];
-						$item["email_address"] = $user_data[PR_EMAIL_ADDRESS];
-						$item["address_type"] = 'ZARAFA';
+					} else {
+						$item['user_name'] = isset($user_data[PR_ACCOUNT]) ? $user_data[PR_ACCOUNT] : $item['display_name'];
+						$item['display_type_ex'] = isset($user_data[PR_DISPLAY_TYPE_EX]) ? $user_data[PR_DISPLAY_TYPE_EX] : MAPI_MAILUSER;
+						$item['email_address'] = isset($user_data[PR_EMAIL_ADDRESS]) ? $user_data[PR_EMAIL_ADDRESS] : '';
+						$item['smtp_address'] = isset($user_data[PR_SMTP_ADDRESS]) ? $user_data[PR_SMTP_ADDRESS] : $item['email_address'];
+						$item['address_type'] = isset($user_data[PR_ADDRTYPE]) ? $user_data[PR_ADDRTYPE] : 'SMTP';
+					}
+
+					if (isset($user_data[PR_SEARCH_KEY])) {
+						$item['search_key'] = bin2hex($user_data[PR_SEARCH_KEY]);
+					} else {
+						$item['search_key'] = bin2hex(strtoupper($item['address_type'] . ':' . $item['smtp_address'])) . '00';
 					}
 
 					array_push($items, $item);
@@ -254,11 +271,11 @@
 			// Open the AB Root Container by not supplying an entryid
 			$abRootContainer = mapi_ab_openentry($ab);
 
-			// Get the "Zarafa Contact Folders"
+			// Get the 'Zarafa Contact Folders'
 			$hierarchyTable = mapi_folder_gethierarchytable($abRootContainer, MAPI_DEFERRED_ERRORS);
 			$abHierarchyRows = mapi_table_queryallrows($hierarchyTable, array(PR_DISPLAY_NAME, PR_AB_PROVIDER_ID, PR_ENTRYID));
 
-			// Look for the "Zarafa Contacts Folders"
+			// Look for the 'Zarafa Contacts Folders'
 			$contactFolderContainerEntryid = false;
 			for($i=0,$len=count($abHierarchyRows);$i<$len;$i++){
 				// Check if the folder matches the Contact Provider GUID
@@ -268,14 +285,14 @@
 				}
 			}
 
-			// Next go into the "Zarafa Contacts Folders" and look in the hierarchy table for the Contact folders.
+			// Next go into the 'Zarafa Contacts Folders' and look in the hierarchy table for the Contact folders.
 			if($abContactContainerEntryid){
-				// Get the rows from hierarchy table of the "Zarafa Contacts Folders"
+				// Get the rows from hierarchy table of the 'Zarafa Contacts Folders'
 				$abContactContainer = mapi_ab_openentry($ab, $abContactContainerEntryid);
 				$abContactContainerHierarchyTable = mapi_folder_gethierarchytable($abContactContainer, MAPI_DEFERRED_ERRORS);
 				$abContactContainerHierarchyRows = mapi_table_queryallrows($abContactContainerHierarchyTable, array(PR_DISPLAY_NAME, PR_OBJECT_TYPE, PR_ENTRYID));
 
-				// Loop through all the contact folders found under the "Zarafa Contacts Folders" hierarchy
+				// Loop through all the contact folders found under the 'Zarafa Contacts Folders' hierarchy
 				for($j=0,$len=count($abContactContainerHierarchyRows);$j<$len;$j++){
 					
 					// Open, get contents table, restrict, sort and then merge the result in the list of $rows
@@ -357,5 +374,35 @@
 
 			return $restriction;
 		}
+
+		/**
+		 * Function does customization of exception based on module data.
+		 * like, here it will generate display message based on actionType
+		 * for particular exception.
+		 * 
+		 * @param object $e Exception object
+		 * @param string $actionType the action type, sent by the client
+		 * @param MAPIobject $store Store object of message.
+		 * @param string $parententryid parent entryid of the message.
+		 * @param string $entryid entryid of the message.
+		 * @param array $action the action data, sent by the client
+		 */
+		function handleException(&$e, $actionType = null, $store = null, $parententryid = null, $entryid = null, $action = null)
+		{
+			if(is_null($e->displayMessage)) {
+				switch($actionType)
+				{
+					case 'checknames':
+						if($e->getCode() == MAPI_E_NO_ACCESS) {
+							$e->setDisplayMessage(_('You have insufficient privileges to perform this action.'));
+						} else {
+							$e->setDisplayMessage(_('Could not resolve user.'));
+						}
+						break;
+				}
+			}
+
+			parent::handleException($e, $actionType, $store, $parententryid, $entryid, $action);
+		}
 	}
 ?>
