@@ -48,7 +48,7 @@ class suggestEmailAddressModule extends Module
 			{
 				if(isset($actionType)) {
 					switch($actionType) {
-						case 'deleteRecipient':
+						case 'delete':
 							$this->deleteRecipient($action, $recipient_history);
 							break;
 						case 'list':
@@ -90,31 +90,29 @@ class suggestEmailAddressModule extends Module
 	 * @param {Array} $recipient_history recipient history stored in mapi proeprty.
 	 */
 	function deleteRecipient($action, $recipient_history) {
-		if(isset($action['deleteRecipient'])  && !empty($recipient_history) && !empty($recipient_history['recipients'])) {
-			$l_aEmailAddresses = explode(";", $action['deleteRecipient']);
-
-			for($i = 0, $len = count($l_aEmailAddresses); $i < $len; $i++){
-				/**
-				 * A foreach is used instead of a normal for-loop to
-				 * prevent the loop from finishing before the end of
-				 * the array, because of the unsetting of elements 
-				 * in that array.
-				 **/
-				foreach($recipient_history['recipients'] as $index => $recipient){
-					if($l_aEmailAddresses[$i] == $recipient['email_address']){
-						unset($recipient_history['recipients'][$index]);
-					}
+		if(isset($action)  && !empty($recipient_history) && !empty($recipient_history['recipients'])) {
+			/**
+			 * A foreach is used instead of a normal for-loop to
+			 * prevent the loop from finishing before the end of
+			 * the array, because of the unsetting of elements 
+			 * in that array.
+			 **/
+			foreach($recipient_history['recipients'] as $index => $recipient){
+				if($action['email_address'] == $recipient['email_address'] || $action['smtp_address'] == $recipient['smtp_address']){
+					unset($recipient_history['recipients'][$index]);
 				}
-
-				// Write new recipient history to property
-				$l_sNewRecipientHistoryJSON = JSON::Encode($recipient_history);
-
-				$stream = mapi_openproperty($GLOBALS["mapisession"]->getDefaultMessageStore(), PR_EC_RECIPIENT_HISTORY_JSON, IID_IStream, 0, MAPI_CREATE | MAPI_MODIFY);
-				mapi_stream_setsize($stream, strlen($l_sNewRecipientHistoryJSON));
-				mapi_stream_write($stream, $l_sNewRecipientHistoryJSON);
-				mapi_stream_commit($stream);
-				mapi_savechanges($GLOBALS["mapisession"]->getDefaultMessageStore());
 			}
+			// Re-indexing recipients' array to adjust index of deleted recipients
+			$recipient_history['recipients'] = array_values($recipient_history['recipients']);
+			
+			// Write new recipient history to property
+			$l_sNewRecipientHistoryJSON = JSON::Encode($recipient_history);
+
+			$stream = mapi_openproperty($GLOBALS["mapisession"]->getDefaultMessageStore(), PR_EC_RECIPIENT_HISTORY_JSON, IID_IStream, 0, MAPI_CREATE | MAPI_MODIFY);
+			mapi_stream_setsize($stream, strlen($l_sNewRecipientHistoryJSON));
+			mapi_stream_write($stream, $l_sNewRecipientHistoryJSON);
+			mapi_stream_commit($stream);
+			mapi_savechanges($GLOBALS["mapisession"]->getDefaultMessageStore());
 		}
 
 		// send success message to client
@@ -211,6 +209,7 @@ class suggestEmailAddressModule extends Module
 
 				$recipientList = Array();
 				foreach($l_aSortedList as $index => $recipient) {
+					$recipient['id'] = count($recipientList) + 1;
 					$recipientList[] = $recipient;
 				}
 			}
