@@ -54,10 +54,12 @@
 				}
 
 				// Go over all files and register them
+				$returnfiles = array();
+
 				foreach($FILES as $FILE) {
 					// Parse the filename, strip it from
 					// any illegal characters.
-					$filename = basename(stripslashes($FILE["name"]));
+					$filename = mb_basename(stripslashes($FILE["name"]));
 
 					// Move the uploaded file into the attachment state
 					$attachid = $attachment_state->addUploadedAttachmentFile($_REQUEST["dialog_attachments"], $filename, $FILE["tmp_name"], array(
@@ -68,11 +70,35 @@
 					));
 
 					$newfiles[] = $attachid;
+					$returnfiles[] = Array(
+						'props' => Array(
+							'attach_num' => -1,
+							'tmpname' => $attachid,
+							'name' => $filename,
+							'size' => $FILE["size"]
+						)
+					);
 				}
+
+				$return = Array(
+					// 'success' property is needed for Extjs Ext.form.Action.Submit#success handler
+					'success' => true,
+					'zarafa' => Array(
+						get('module', false, false, STRING_REGEX) => Array(
+							get('moduleid', false, false, STRING_REGEX) => Array(
+								'update' => Array(
+									'item'=> $returnfiles
+								)
+							)
+						)
+					)
+				);
+
+				echo JSON::Encode($return);
 			} else if(isset($_POST["deleteattachment"]) && isset($_POST["type"])) { // Delete uploaded file
 				// Parse the filename, strip it from
 				// any illegal characters
-				$filename = basename(stripslashes(urldecode($_POST["deleteattachment"])));
+				$filename = mb_basename(stripslashes(urldecode($_POST["deleteattachment"])));
 
 				// Check if the delete file is an uploaded or an attachment of a MAPI iMessage
 				if ($_POST["type"] == "new") {
@@ -82,6 +108,22 @@
 					// Set the correct array structure
 					$attachment_state->addDeletedAttachment($_REQUEST["dialog_attachments"], $filename);
 				}
+
+				$return = Array(
+					// 'success' property is needed for Extjs Ext.form.Action.Submit#success handler
+					'success' => true,
+					'zarafa' => Array(
+						get('module', false, false, STRING_REGEX) => Array(
+							get('moduleid', false, false, STRING_REGEX) => Array(
+								'delete' => Array(
+									'success'=> true
+								)
+							)
+						)
+					)
+				);
+
+				echo JSON::Encode($return);
 			}
 		} else if($_GET && isset($_GET["attachment_id"])) { // this is to upload the file to server when the doc is send via OOo
 
@@ -89,7 +131,7 @@
 
 			// check wheather the doc is already moved
 			if (file_exists($providedFile)) {
-				$filename = basename(stripslashes($_GET["name"]));
+				$filename = mb_basename(stripslashes($_GET["name"]));
 
 				// Move the uploaded file to the session
 				$attachment_state->addProvidedAttachmentFile($_REQUEST["attachment_id"], $filename, $providedFile, array(
@@ -103,29 +145,6 @@
 				$attachment_state->clearAttachmentFiles($_GET["attachment_id"]);
 			}
 		}
-
-		// Return the file data output when client has request this page through the load==upload_attachment
-		if(get('load', false, false, STRING_REGEX) == 'upload_attachment'){
-			$return = Array(
-				'success'=>true,
-				'files'=> Array()
-			);
-
-			$files = $attachment_state->getAttachmentFiles($_REQUEST["dialog_attachments"]);
-			if ($files) {
-				foreach($files as $tmpname => $file){
-					$newfileIndicator = in_array($tmpname, $newfiles);
-					$return['files'][] = Array(
-						'tmpname' => $tmpname,
-						'filename' => $file["name"],
-						'size' => $file["size"],
-						'newfile' => $newfileIndicator
-					);
-				}
-			}
-
-			echo JSON::Encode($return);
-		}
 	} catch (ZarafaErrorException $e) {
 		/**
 		 * Return Exception message only if uploading attachment is done through
@@ -133,13 +152,20 @@
 		 */
 		if(get('load', false, false, STRING_REGEX) == 'upload_attachment'){
 			$return = Array(
-				'success'=>false,
-				'error' => array(
-					"type" => ERROR_GENERAL,
-					"info" => array(
-						"file" => $e->getFileLine(),
-						"display_message" => _("Could not upload attachment."),
-						"original_message" => $e->getMessage()
+				// 'success' property is needed for Extjs Ext.form.Action.Submit#success handler
+				'success' => false,
+				'zarafa' => Array(
+					get('module', false, false, STRING_REGEX) => Array(
+						get('moduleid', false, false, STRING_REGEX) => Array(
+							'error' => array(
+								"type" => ERROR_GENERAL,
+								"info" => array(
+									"file" => $e->getFileLine(),
+									"display_message" => _("Could not upload attachment."),
+									"original_message" => $e->getMessage()
+								)
+							)
+						)
 					)
 				)
 			);
