@@ -406,6 +406,34 @@
 		}
 	}
 
+	/**
+	 * This function will return base name of the file from the full path of the file. 
+	 * PHP's basename() does not properly support streams or filenames beginning with a non-US-ASCII character.
+	 * The default implementation in php for basename is locale aware. so it will truncate umlauts which can not be
+	 * parsed by the current set locale.
+	 * This problem only occurs with PHP < 5.2
+	 * @see http://bugs.php.net/bug.php?id=37738, https://bugs.php.net/bug.php?id=37268
+	 * @param String $filepath full path of the file
+	 * @param String $suffix suffix that will be trimmed from file name
+	 * @return String base name of the file
+	 */
+	function mb_basename($filepath, $suffix = '')
+	{
+		// Remove right-most slashes when $uri points to directory.
+		$filepath = rtrim($filepath, DIRECTORY_SEPARATOR . ' ');
+
+		// Returns the trailing part of the $uri starting after one of the directory
+		// separators.
+		$filename = preg_match('@[^' . preg_quote(DIRECTORY_SEPARATOR, '@') . ']+$@', $filepath, $matches) ? $matches[0] : '';
+
+		// Cuts off a suffix from the filename.
+		if ($suffix) {
+			$filename = preg_replace('@' . preg_quote($suffix, '@') . '$@', '', $filename);
+		}
+
+		return $filename;
+	}
+
 	// Constants for regular expressions which are used in get method to verify the input string
 	define("ID_REGEX", "/^[a-z0-9_]+$/im");
 	define("STRING_REGEX", "/^[a-z0-9_\s()@]+$/im");
