@@ -57,21 +57,30 @@
 	session_start();
 	
 	// Create global mapi object. This object is used in many other files
-	$GLOBALS["mapisession"] = new MAPISession();
-	// Logon, the username and password are set in the "index.php" file. So whenever
-	// an user enters this file, the username and password whould be set in the $_SESSION
-	// variable
-	if (isset($_SESSION["username"]) && isset($_SESSION["password"])) {
-		$sslcert_file = defined('SSLCERT_FILE') ? SSLCERT_FILE : null;
-		$sslcert_pass = defined('SSLCERT_PASS') ? SSLCERT_PASS : null;
-		$hresult = $GLOBALS["mapisession"]->logon($_SESSION["username"], $_SESSION["password"], DEFAULT_SERVER, $sslcert_file, $sslcert_pass);
-	}else{
-		$hresult = MAPI_E_UNCONFIGURED;
+	$GLOBALS["mapisession"] = new MAPISession(session_id());
+
+	// We will only allow the logon when the sessionid in the GET arguments matches the
+	// sessionid as send in the cookie. Otherwise the cookie was somehow modified in the
+	// browser.
+	$sessionid = get('sessionid', '', false, ID_REGEX);
+	if ($sessionid === $GLOBALS["mapisession"]->getSessionId()) {
+		// Logon, the username and password are set in the "index.php" file. So whenever
+		// an user enters this file, the username and password whould be set in the $_SESSION
+		// variable
+		if (isset($_SESSION["username"]) && isset($_SESSION["password"])) {
+			$sslcert_file = defined('SSLCERT_FILE') ? SSLCERT_FILE : null;
+			$sslcert_pass = defined('SSLCERT_PASS') ? SSLCERT_PASS : null;
+			$hresult = $GLOBALS["mapisession"]->logon($_SESSION["username"], $_SESSION["password"], DEFAULT_SERVER, $sslcert_file, $sslcert_pass);
+		} else {
+			$hresult = MAPI_E_UNCONFIGURED;
+		}
+	} else {
+		$hresult = MAPI_E_INVALID_WORKSTATION_ACCOUNT;
 	}
 
-	if(isset($_SESSION["lang"])) {
+	if (isset($_SESSION["lang"])) {
 		$session_lang = $_SESSION["lang"];
-	}else{
+	} else {
 		$session_lang = LANG;
 	}
 	
@@ -86,8 +95,47 @@
 	header("Pragma: no-cache");
 	header("X-Zarafa: " . trim(file_get_contents('version')));
 
-	// Check is the user is authenticated
-	if ($GLOBALS["mapisession"]->isLoggedOn()) {
+	if (isset($_GET['ping'])) {
+		$pingTag = array(
+			'info' => array(
+				'hresult' => $hresult,
+				'hresult_name' => get_mapi_error_name($hresult)
+			)
+		);
+
+		switch ($hresult) {
+			// A network error indicates that the connection
+			// to the server could not be established.
+			case MAPI_E_NETWORK_ERROR:
+				$pingTag['success'] = false;
+				break;
+			// The following errors are specific to logging on,
+			// it means the connection to the server did work,
+			// but the user simply isn't logged in.
+			case MAPI_E_PASSWORD_CHANGE_REQUIRED:
+			case MAPI_E_PASSWORD_EXPIRED:
+			case MAPI_E_INVALID_WORKSTATION_ACCOUNT:
+			case MAPI_E_INVALID_ACCESS_TIME:
+			case MAPI_E_ACCOUNT_DISABLED:
+			// Error name says it all, connection exists, but cannot logon.
+			case MAPI_E_LOGON_FAILED:
+			// MAPI_E_UNCONFIGURED means we didn't try to connect to the server,
+			// but the user is not logged in, so the state has changed for the user.
+			case MAPI_E_UNCONFIGURED:
+			// NOERROR means the logon was successfull, and the connection to the
+			// server is working as expected.
+			case NOERROR:
+			// All other errors are considered a 'success' for connecting to the
+			// server and the 'active' property will indicate if the user still
+			// has an active session to the server.
+			default:
+				$pingTag['success'] = true;
+				$pingTag['active'] = $GLOBALS["mapisession"]->isLoggedOn();
+				break;
+		}
+
+		echo json_encode($pingTag);
+	} else if ($GLOBALS["mapisession"]->isLoggedOn()) {
 		// Authenticated
 		// Execute request
 		
@@ -111,10 +159,7 @@
 		$GLOBALS["language"]->setLanguage($session_lang);
 
 		// Get the state information for this subsystem
-		if(isset($_GET["subsystem"]))
-			$subsystem = $_GET["subsystem"];
-		else
-			$subsystem = "anonymous"; // Currently should never happen	
+		$subsystem = get('subsystem', 'anonymous', false, ID_REGEX);
 
 		$state = new State($subsystem);
 		
@@ -194,20 +239,15 @@
 		// You can skip this as well because the lock is freed after the PHP script ends
 		// anyway. (only for PHP < 5.3.2)
 		$state->close();
+	} else if ($hresult === MAPI_E_NETWORK_ERROR) {
+		// The user is not logged in because the zarafa-server could not be reached.
+		// Return a HTTP 503 error so the client can act upon this event correctly.
+		header('HTTP/1.1 503 Service unavailable');
+		header("X-Zarafa-Hresult: " . get_mapi_error_name($hresult));
 	} else {
-		$errorTag = array(
-			"zarafa" => array(
-				"error" => array(
-					"type" => ERROR_MAPI,
-					"info" => array(
-						"hresult" => $hresult,
-						"hresult_name" => get_mapi_error_name($hresult),
-						"display_message" => _("Logon failed")
-					)
-				)
-			)
-		);
-
-		echo json_encode($errorTag);
+		// The session expired, or the user is otherwise not logged on.
+		// Return a HTTP 401 error so the client can act upon this event correctly.
+		header('HTTP/1.1 401 Unauthorized');
+		header("X-Zarafa-Hresult: " . get_mapi_error_name($hresult));
 	}
 ?>
